package me.trixkz.lifestealcore;

import aether.Aether;
import lombok.Getter;
import me.trixkz.lifestealcore.listeners.*;
import me.trixkz.lifestealcore.managers.*;
import me.trixkz.lifestealcore.managers.chatgames.ChatGameManager;
import me.trixkz.lifestealcore.managers.clans.ClanManager;
import me.trixkz.lifestealcore.managers.envoys.EnvoyManager;
import me.trixkz.lifestealcore.managers.killstreaks.KillstreakManager;
import me.trixkz.lifestealcore.managers.koth.KOTHManager;
import me.trixkz.lifestealcore.managers.leaderboards.LeaderboardManager;
import me.trixkz.lifestealcore.managers.pvpclass.PvPClassManager;
import me.trixkz.lifestealcore.normalmenusystem.PlayerMenuUtil;
import me.trixkz.lifestealcore.playerdata.PlayerData;
import me.trixkz.lifestealcore.runnables.*;
import me.trixkz.lifestealcore.scoreboard.ScoreboardProvider;
import me.trixkz.lifestealcore.utils.ColorUtils;
import me.trixkz.lifestealcore.utils.Constants;
import me.trixkz.lifestealcore.utils.Utils;
import me.trixkz.lifestealcore.utils.config.FileConfig;
import me.trixkz.lifestealcore.utils.config.file.Config;
import net.luckperms.api.LuckPerms;
import org.bukkit.Bukkit;
import org.bukkit.Chunk;
import org.bukkit.World;
import org.bukkit.entity.Entity;
import org.bukkit.entity.EntityType;
import org.bukkit.entity.Player;
import org.bukkit.plugin.RegisteredServiceProvider;
import org.bukkit.plugin.java.JavaPlugin;
import java.util.*;

@Getter
public class LifeStealCore extends JavaPlugin {

    @Getter public static LifeStealCore instance;

    private Config settingsConfig;
    private FileConfig messagesConfig;
    private FileConfig kothsConfig;
    private FileConfig lootboxConfig;
    private FileConfig envoysConfig;

    private CommandManager commandManager;
    private ClanManager clanManager;
    private ChunkManager chunkManager;
    private PvPClassManager pvpClassManager;
    private KOTHManager kothManager;
    private SpawnManager spawnManager;
    private MongoManager mongoManager;
    private PlayerDataManager playerDataManager;
    private LootboxManager lootboxManager;
    private LeaderboardManager leaderboardManager;
    private ChatGameManager chatGameManager;
    private CombatManager combatManager;
    private CooldownManager cooldownManager;
    private KillstreakManager killstreakManager;
    private EnvoyManager envoyManager;

    private LuckPerms luckPerms;

    private HashMap<Player, PlayerMenuUtil> playerMenuUtilMap = new HashMap<Player, PlayerMenuUtil>();

    private List<UUID> players = new ArrayList<UUID>();

    public void onEnable() {
        instance = this;

        this.saveDefaultConfig();
        this.settingsConfig = new Config("config", this);
        this.messagesConfig = new FileConfig(this, "messages.yml");
        this.kothsConfig = new FileConfig(this, "koths.yml");
        this.lootboxConfig = new FileConfig(this, "lootbox.yml");
        this.envoysConfig = new FileConfig(this, "envoys.yml");

        Bukkit.getConsoleSender().sendMessage("------------------------------------------------");
        Bukkit.getConsoleSender().sendMessage(Utils.translate("&dLifestealCore &8- &av" + this.getDescription().getVersion()));
        Bukkit.getConsoleSender().sendMessage(Utils.translate("&7Made by &eLoganM Development"));
        Bukkit.getConsoleSender().sendMessage("------------------------------------------------");

        new Constants();
        new ColorUtils();

        this.loadManagers();
        this.loadListeners();
        this.loadRunnables();

        for (World world : Bukkit.getWorlds()) {
            for (Entity entity : world.getEntities()) {
                if (entity.getType() != EntityType.PLAYER && entity.getType() != EntityType.ITEM_FRAME) {
                    entity.remove();
                }
            }

            world.setGameRuleValue("doDaylightCycle", "false");
            world.setTime(0L);
            world.setStorm(false);
        }

        RegisteredServiceProvider<LuckPerms> provider = Bukkit.getServicesManager().getRegistration(LuckPerms.class);

        if (provider != null) {
            luckPerms = provider.getProvider();
        }
    }

    public void onDisable() {
        for (Player player : this.getServer().getOnlinePlayers()) {
            PlayerData playerData = this.playerDataManager.getPlayerData(player.getUniqueId());

            this.playerDataManager.savePlayerData(playerData);
        }

        this.getServer().getScheduler().cancelTask(this.leaderboardManager.getLeaderboard().getHologramTaskId());
        this.envoyManager.saveEnvoyLocations();
        this.mongoManager.disconnect();

        for (World world : Bukkit.getWorlds()) {
            for (Entity entity : world.getEntities()) {
                if (entity.getType() == EntityType.DROPPED_ITEM) {
                    entity.remove();
                }
            }

            for (Chunk chunk : world.getLoadedChunks()) {
                chunk.unload(true);
            }
        }

        instance = null;
    }

    private void loadManagers() {
        this.commandManager = new CommandManager();
        this.clanManager = new ClanManager();
        this.chunkManager = new ChunkManager();
        this.pvpClassManager = new PvPClassManager();
        this.kothManager = new KOTHManager();
        this.spawnManager = new SpawnManager();
        this.mongoManager = new MongoManager();
        this.playerDataManager = new PlayerDataManager();
        this.lootboxManager = new LootboxManager();
        this.leaderboardManager = new LeaderboardManager();
        this.chatGameManager = new ChatGameManager();
        this.combatManager = new CombatManager();
        this.cooldownManager = new CooldownManager();
        this.killstreakManager = new KillstreakManager();
        this.envoyManager = new EnvoyManager();
    }

    private void loadListeners() {
        Arrays.asList(
                new MenuListener(),
                new ButtonListener(),
                new ClanListeners(),
                this.pvpClassManager,
                new EventListeners(),
                new KOTHListener(),
                new RandomListeners(),
                new PlayerDataListener(),
                new BlockBreakListener(),
                new BlockExplodeListener(),
                new BlockPlaceListener(),
                new BlockSpreadListener(),
                new BucketListener(),
                new EntityChangeBlockListener(),
                new PlayerDeathListener(),
                new EntityDamageByEntityListener(),
                new EntityDeathListener()
        ).forEach(listener -> this.getServer().getPluginManager().registerEvents(listener, this));
    }

    private void loadRunnables() {
        new Aether(this, new ScoreboardProvider());

        new SpawnRunnable();
        new ClearLagRunnable();
        new CombatTagRunnable();
        new EnvoyRunnable();
    }

    public PlayerMenuUtil getPlayerMenuUtil(Player player) {
        PlayerMenuUtil playerMenuUtil;

        if (playerMenuUtilMap.containsKey(player)) {
            return playerMenuUtilMap.get(player);
        } else {
            playerMenuUtil = new PlayerMenuUtil(player);

            playerMenuUtilMap.put(player, playerMenuUtil);

            return playerMenuUtil;
        }
    }
}
