package me.trixkz.lifestealcore.advancedmenusystem.shop;

import lombok.AllArgsConstructor;
import lombok.Getter;
import lombok.RequiredArgsConstructor;
import lombok.Setter;
import me.trixkz.lifestealcore.LifeStealCore;
import me.trixkz.lifestealcore.advancedmenusystem.Button;
import me.trixkz.lifestealcore.playerdata.PlayerData;
import me.trixkz.lifestealcore.utils.Utils;
import org.bukkit.Material;
import org.bukkit.entity.Player;
import org.bukkit.event.inventory.ClickType;
import org.bukkit.inventory.ItemStack;
import org.bukkit.inventory.meta.ItemMeta;
import java.util.ArrayList;
import java.util.List;

@Getter
@Setter
@AllArgsConstructor
@RequiredArgsConstructor
public class ShopItemButton extends Button {

    private ItemStack itemStack;
    private int buyPrice;
    private int sellPrice;
    private boolean buy;
    private ShopItemType shopItemType;

    @Override
    public ItemStack getButtonItem(Player player) {
        PlayerData playerData = LifeStealCore.getInstance().getPlayerDataManager().getPlayerData(player.getUniqueId());

        if (this.buy) {
            this.buyPrice *= this.itemStack.getAmount();
        } else {
            this.sellPrice *= this.itemStack.getAmount();
        }

        ItemStack itemStackCopy = new ItemStack(this.itemStack.getType());

        ItemMeta itemMeta = itemStackCopy.getItemMeta();

        List<String> lore = new ArrayList<String>();

        switch (this.shopItemType) {
            case ITEM:
                ItemMeta itemMetaCopy = itemStackCopy.getItemMeta();
                itemMetaCopy.setDisplayName(Utils.translate("&d" + this.itemStack.getType().name()));

                if (this.buy) {
                    lore.add("&9Buy price: &d" + this.buyPrice);
                } else {
                    lore.add("&9Sell price: &d" + this.sellPrice);
                }

                itemMetaCopy.setLore(Utils.translate(lore));

                this.itemStack.setItemMeta(itemMetaCopy);

                break;
            case CONFIRM:
                if (this.buy) {
                    lore.add(playerData.getCoins() >= this.buyPrice ? "&aClick to buy" : "&cNot enough coins");
                } else {
                    int itemStacks = 0;

                    for (ItemStack itemStack : player.getInventory().getContents()) {
                        if (itemStack == null) {
                            continue;
                        }

                        if (this.itemStack.getType() == itemStack.getType()) {
                            itemStacks += itemStack.getAmount();
                        }
                    }

                    lore.add(itemStacks >= this.itemStack.getAmount() ? "&aClick to sell" : "&cNot enough items");
                }

                itemStackCopy.setType(Material.GREEN_STAINED_GLASS);

                itemMeta.setDisplayName(Utils.translate("&aConfirm"));
                itemMeta.setLore(Utils.translate(lore));

                break;
            case CANCEL:
                itemStackCopy.setType(Material.RED_STAINED_GLASS);

                itemMeta.setDisplayName(Utils.translate("&cCancel"));

                break;
            case ADD_ONE:
                itemStackCopy.setAmount(1);
                itemStackCopy.setType(Material.GREEN_STAINED_GLASS_PANE);

                itemMeta.setDisplayName(Utils.translate("&aAdd 1"));

                break;
            case ADD_TEN:
                itemStackCopy.setAmount(10);
                itemStackCopy.setType(Material.GREEN_STAINED_GLASS_PANE);

                itemMeta.setDisplayName(Utils.translate("&aAdd 10"));

                break;
            case SET_TO_SIXTY_FOUR:
                itemStackCopy.setAmount(64);
                itemStackCopy.setType(Material.GREEN_STAINED_GLASS_PANE);

                itemMeta.setDisplayName(Utils.translate("&aSet to 64"));

                break;
            case REMOVE_ONE:
                itemStackCopy.setAmount(1);
                itemStackCopy.setType(Material.RED_STAINED_GLASS_PANE);

                itemMeta.setDisplayName(Utils.translate("&cRemove 1"));

                break;
            case REMOVE_TEN:
                itemStackCopy.setAmount(10);
                itemStackCopy.setType(Material.RED_STAINED_GLASS_PANE);

                itemMeta.setDisplayName(Utils.translate("&cRemove 10"));

                break;
            case SET_TO_ONE:
                itemStackCopy.setAmount(1);
                itemStackCopy.setType(Material.RED_STAINED_GLASS_PANE);

                itemMeta.setDisplayName(Utils.translate("&cSet to 1"));

                break;
            case MORE:
                itemStackCopy.setAmount(this.itemStack.getAmount() + 64);

                if (this.buy) {
                    itemStackCopy.setType(Material.GREEN_STAINED_GLASS_PANE);
                } else {
                    itemStackCopy.setType(Material.RED_STAINED_GLASS_PANE);
                }

                itemMeta.setDisplayName(Utils.translate(this.buy ? "&aBuy more" : "&cSell more"));

                break;
        }

        itemStackCopy.setItemMeta(itemMeta);

        return this.shopItemType == ShopItemType.ITEM ? this.itemStack : itemStackCopy;
    }

    @Override
    public void clicked(Player player, int slot, ClickType clickType, int hotbarButton) {
        PlayerData playerData = LifeStealCore.getInstance().getPlayerDataManager().getPlayerData(player.getUniqueId());

        switch (this.shopItemType) {
            case ITEM:
                break;
            case CONFIRM:
                if (this.buy) {
                    if (playerData.getCoins() >= this.buyPrice) {
                        player.getInventory().addItem(new ItemStack(this.itemStack.getType(), this.itemStack.getAmount()));

                        playerData.setCoins(playerData.getCoins() - this.buyPrice);
                    }
                } else {
                    int itemStacks = 0;

                    for (ItemStack itemStack : player.getInventory().getContents()) {
                        if (itemStack == null) {
                            continue;
                        }

                        if (this.itemStack.getType() == itemStack.getType()) {
                            itemStacks += itemStack.getAmount();
                        }
                    }

                    if (itemStacks >= this.itemStack.getAmount()) {
                        player.getInventory().removeItem(new ItemStack(this.itemStack.getType(), this.itemStack.getAmount()));

                        playerData.setCoins(playerData.getCoins() + this.itemStack.getAmount());
                    }
                }

                break;
            case CANCEL:
                player.closeInventory();

                break;
            case ADD_ONE:
                this.itemStack.setAmount(this.itemStack.getAmount() + 1);

                break;
            case ADD_TEN:
                this.itemStack.setAmount(this.itemStack.getAmount() + 10);

                break;
            case SET_TO_SIXTY_FOUR:
                this.itemStack.setAmount(64);

                break;
            case REMOVE_ONE:
                this.itemStack.setAmount(this.itemStack.getAmount() - 1);

                if (this.itemStack.getAmount() < 1) {
                    this.itemStack.setAmount(1);
                }

                break;
            case REMOVE_TEN:
                this.itemStack.setAmount(this.itemStack.getAmount() - 10);

                if (this.itemStack.getAmount() < 1) {
                    this.itemStack.setAmount(1);
                }

                break;
            case SET_TO_ONE:
                this.itemStack.setAmount(1);

                break;
            case MORE:
                break;
        }

        player.updateInventory();
    }
}
