package me.trixkz.lifestealcore.commands.coins;

import me.trixkz.lifestealcore.LifeStealCore;
import me.trixkz.lifestealcore.commands.BaseCommand;
import me.trixkz.lifestealcore.commands.CommandInfo;
import me.trixkz.lifestealcore.playerdata.PlayerData;
import me.trixkz.lifestealcore.utils.Constants;
import me.trixkz.lifestealcore.utils.Utils;
import org.bukkit.command.CommandSender;
import org.bukkit.entity.Player;
import java.text.DecimalFormat;
import java.util.ArrayList;

@CommandInfo(name = "coins", permission = "", player = true)
public class CoinsCommand extends BaseCommand {

    private LifeStealCore main = LifeStealCore.getInstance();

    private ArrayList<String> message = new ArrayList<String>();

    @Override
    public void executeAs(Player player, String[] args) {
        PlayerData playerData = this.main.getPlayerDataManager().getPlayerData(player.getUniqueId());

        DecimalFormat decimalFormat = new DecimalFormat("##.##");

        if (args.length == 0) {
            player.sendMessage(Utils.translate(Constants.PREFIX + this.main.getMessagesConfig().getConfig().getString("COINS.BALANCE").replace("%coins%", decimalFormat.format(playerData.getCoins()))));

            return;
        } else {
            switch (args[0].toLowerCase()) {
                case "help":
                    for (String string : this.main.getMessagesConfig().getConfig().getStringList("COINS.HELP_MESSAGE.PLAYER")) {
                        this.message.add(string.replace("%line%", Utils.chatBar));
                    }

                    if (player.hasPermission("skyblock.admin")) {
                        for (String string : this.main.getMessagesConfig().getConfig().getStringList("COINS.HELP_MESSAGE.ADMIN")) {
                            this.message.add(string.replace("%line%", Utils.chatBar));
                        }
                    }

                    for (String string : this.message) {
                        player.sendMessage(Utils.translate(string));
                    }

                    this.message.clear();

                    return;
                case "pay":
                    Player payTarget = this.main.getServer().getPlayer(args[1]);

                    if (payTarget == null) {
                        return;
                    }

                    PlayerData payTargetPlayerData = this.main.getPlayerDataManager().getPlayerData(payTarget.getUniqueId());

                    if (!Utils.isNumeric(args[2])) {
                        return;
                    }

                    double payAmount = Double.parseDouble(args[2]);

                    if (playerData.getCoins() >= payAmount) {
                        playerData.setCoins(playerData.getCoins() - payAmount);
                        payTargetPlayerData.setCoins(payTargetPlayerData.getCoins() + payAmount);
                    }

                    player.sendMessage(Utils.translate("&aPaid &a&l" + payTarget.getName() + "&a's &a&l" + decimalFormat.format(payAmount) + " &acoins"));
                    payTarget.sendMessage(Utils.translate("&a&l" + player.getName() + " &ahas paid you &a&l" + decimalFormat.format(payAmount) + " &acoins"));

                    return;
                case "set":
                case "add":
                case "remove":
                    if (!player.hasPermission("skyblock.admin")) {
                        player.sendMessage(Utils.translate("&cNo permission"));

                        return;
                    }

                    Player target = this.main.getServer().getPlayer(args[1]);

                    if (target == null) {
                        return;
                    }

                    PlayerData targetPlayerData = this.main.getPlayerDataManager().getPlayerData(target.getUniqueId());

                    if (!Utils.isNumeric(args[2])) {
                        return;
                    }

                    double amount = Double.parseDouble(args[2]);

                    switch (args[0].toLowerCase()) {
                        case "set":
                            targetPlayerData.setCoins(amount);

                            player.sendMessage(Utils.translate("&aSet &a&l" + target.getName() + " &acoins balance to &a&l" + decimalFormat.format(amount)));
                            target.sendMessage(Utils.translate("&aCoins balance updated, you now have &a&l" + decimalFormat.format(targetPlayerData.getCoins()) + " &acoins"));

                            break;
                        case "add":
                            targetPlayerData.setCoins(targetPlayerData.getCoins() + amount);

                            player.sendMessage(Utils.translate("&aAdded &a&l" + decimalFormat.format(amount) + " &acoins to &a&l" + target.getName() + "&a's coins balance"));
                            target.sendMessage(Utils.translate("&aCoins balance updated, you now have &a&l" + decimalFormat.format(targetPlayerData.getCoins()) + " &acoins"));

                            break;
                        case "remove":
                            if (targetPlayerData.getCoins() >= amount) {
                                targetPlayerData.setCoins(targetPlayerData.getCoins() - amount);
                            }

                            player.sendMessage(Utils.translate("&aRemoved &a&l" + decimalFormat.format(amount) + " &acoins to &a&l" + target.getName() + "&a's coins balance"));
                            target.sendMessage(Utils.translate("&aCoins balance updated, you now have &a&l" + decimalFormat.format(targetPlayerData.getCoins()) + " &acoins"));

                            break;
                    }

                    break;
            }

            if (!player.hasPermission("skyblock.admin")) {
                player.sendMessage(Utils.translate("&cNo permission"));

                return;
            }

            Player target = this.main.getServer().getPlayer(args[0]);

            if (target == null) {
                return;
            }

            PlayerData targetPlayerData = this.main.getPlayerDataManager().getPlayerData(target.getUniqueId());

            player.sendMessage(Utils.translate("&a&l" + target.getName() + " &ahas &a&l" + decimalFormat.format(targetPlayerData.getCoins()) + " &acoins"));
        }

        return;
    }

    @Override
    public void executeAs(CommandSender sender, String[] args) {

    }
}
