package me.trixkz.lifestealcore.managers;

import io.github.bananapuncher714.nbteditor.NBTEditor;
import lombok.Getter;
import lombok.Setter;
import me.trixkz.lifestealcore.LifeStealCore;
import me.trixkz.lifestealcore.normalmenusystem.ItemStackButton;
import me.trixkz.lifestealcore.utils.Utils;
import org.bukkit.Material;
import org.bukkit.configuration.ConfigurationSection;
import org.bukkit.configuration.file.FileConfiguration;
import org.bukkit.enchantments.Enchantment;
import org.bukkit.entity.Player;
import org.bukkit.inventory.ItemStack;
import org.bukkit.inventory.meta.ItemMeta;
import java.util.*;

@Getter
@Setter
public class LootboxManager {

    private LifeStealCore main = LifeStealCore.getInstance();

    private List<ItemStack> itemStacks = new ArrayList<ItemStack>();

    public LootboxManager() {
        FileConfiguration fileConfiguration = this.main.getLootboxConfig().getConfig();

        ConfigurationSection configurationSection = fileConfiguration.getConfigurationSection("lootbox");

        if (configurationSection == null) {
            return;
        }

        configurationSection.getKeys(false).forEach((number) -> {
            String name = configurationSection.getString(number + ".name");

            List<String> loreArrayList = configurationSection.getStringList(number + ".lore");

            String[] lore = new String[loreArrayList.size()];

            for (int i = 0; i < loreArrayList.size(); i++) {
                lore[i] = loreArrayList.get(i);
            }

            Material material = Material.getMaterial(configurationSection.getString(number + ".material"));

            int data = configurationSection.getInt(number + ".data");
            int amount = configurationSection.getInt(number + ".amount");

            Map<Enchantment, Integer> enchantments = new HashMap<Enchantment, Integer>();

            if (configurationSection.getStringList(number + ".enchantments") != null) {
                configurationSection.getStringList(number + ".enchantments").forEach((string) -> {
                    String[] stringArray = string.split(":");

                    enchantments.put(Enchantment.getByName(stringArray[0]), Integer.parseInt(stringArray[1]));
                });
            }

            ItemStackButton itemStackButton = new ItemStackButton(name, lore, material, data, amount);

            ItemStack itemStack = itemStackButton.makeItemStack();

            if (enchantments.size() >= 1) {
                itemStack.addEnchantments(enchantments);
            }

            this.itemStacks.add(itemStack);
        });
    }

    public ItemStack getLootboxItemStack(Player player, int amount) {
        ItemStack itemStack = new ItemStack(Material.ENDER_CHEST, amount);

        ItemMeta itemMeta = itemStack.getItemMeta();
        itemMeta.setDisplayName(Utils.translate("&dLootbox"));
        itemMeta.setLore(Utils.translate(Arrays.asList("&9Right click to use")));

        itemStack.setItemMeta(itemMeta);
        itemStack = NBTEditor.set(itemStack, "Lootbox", player.getUniqueId().toString() + "Lootbox");

        return itemStack;
    }

    public boolean isLootboxItemStack(Player player, ItemStack itemStack) {
        if (itemStack.getItemMeta() == null) {
            return false;
        }

        return NBTEditor.contains(itemStack, player.getUniqueId().toString() + "Lootbox");
    }
}
