package me.trixkz.lifestealcore.managers;

import com.mongodb.client.MongoCursor;
import com.mongodb.client.model.Filters;
import com.mongodb.client.model.UpdateOptions;
import lombok.Getter;
import me.trixkz.lifestealcore.LifeStealCore;
import me.trixkz.lifestealcore.playerdata.PlayerData;
import me.trixkz.lifestealcore.playerdata.PlayerState;
import me.trixkz.lifestealcore.utils.Constants;
import me.trixkz.lifestealcore.utils.CustomLocation;
import org.bson.Document;
import org.bukkit.entity.Player;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import java.util.UUID;

public class PlayerDataManager {

	private LifeStealCore main = LifeStealCore.getInstance();

	@Getter private Map<UUID, PlayerData> players = new HashMap<>();

	public PlayerData getOrCreate(UUID uniqueId) {
		return this.players.computeIfAbsent(uniqueId, PlayerData::new);
	}

	public PlayerData getPlayerData(UUID uniqueId) {
		return this.players.getOrDefault(uniqueId, new PlayerData(uniqueId));
	}

	public Collection<PlayerData> getAllPlayers() {
		return this.players.values();
	}

	public void loadPlayerData(PlayerData playerData) {
		Document document = this.main.getMongoManager().getPlayers().find(Filters.eq(Constants.PLAYER_DATA_UNIQUE_ID, playerData.getUniqueId().toString())).first();

		if (document != null) {
			playerData.setCoins(document.getDouble(Constants.PLAYER_DATA_COINS));
			playerData.setTokens(document.getInteger(Constants.PLAYER_DATA_TOKENS));
			playerData.setClanOwner((document.getString(Constants.PLAYER_DATA_CLAN_OWNER) == null ? null : UUID.fromString(document.getString(Constants.PLAYER_DATA_CLAN_OWNER))));
			playerData.setKills(document.getInteger(Constants.PLAYER_DATA_KILLS));
			playerData.setDeaths(document.getInteger(Constants.PLAYER_DATA_DEATHS));
			playerData.setHearts(document.getDouble(Constants.PLAYER_DATA_HEARTS));
			playerData.setBounty(document.getInteger(Constants.PLAYER_DATA_BOUNTY));
			playerData.setCurrentKillstreak(document.getInteger(Constants.PLAYER_DATA_CURRENT_KILLSTREAK));
			playerData.setHighestKillstreak(document.getInteger(Constants.PLAYER_DATA_HIGHEST_KILLSTREAK));
			playerData.setDeathBanTime(document.getLong(Constants.PLAYER_DATA_DEATH_BAN_TIME));
			playerData.setDeathBanTimeAddedAt(document.getLong(Constants.PLAYER_DATA_DEATH_BAN_TIME_ADDED_AT));
		}

		playerData.setLoaded(true);
	}

	public void savePlayerData(PlayerData playerData) {
		Document document = new Document();

		document.put(Constants.PLAYER_DATA_UNIQUE_ID, playerData.getUniqueId().toString());
		document.put(Constants.PLAYER_DATA_COINS, playerData.getCoins());
		document.put(Constants.PLAYER_DATA_TOKENS, playerData.getTokens());
		document.put(Constants.PLAYER_DATA_CLAN_OWNER, (playerData.getClanOwner() == null ? null : playerData.getClanOwner().toString()));
		document.put(Constants.PLAYER_DATA_KILLS, playerData.getKills());
		document.put(Constants.PLAYER_DATA_DEATHS, playerData.getDeaths());
		document.put(Constants.PLAYER_DATA_HEARTS, playerData.getHearts());
		document.put(Constants.PLAYER_DATA_BOUNTY, playerData.getBounty());
		document.put(Constants.PLAYER_DATA_CURRENT_KILLSTREAK, playerData.getCurrentKillstreak());
		document.put(Constants.PLAYER_DATA_HIGHEST_KILLSTREAK, playerData.getHighestKillstreak());
		document.put(Constants.PLAYER_DATA_DEATH_BAN_TIME, playerData.getDeathBanTime());
		document.put(Constants.PLAYER_DATA_DEATH_BAN_TIME_ADDED_AT, playerData.getDeathBanTimeAddedAt());

		this.main.getMongoManager().getPlayers().replaceOne(Filters.eq(Constants.PLAYER_DATA_UNIQUE_ID, playerData.getUniqueId().toString()), document, new UpdateOptions().upsert(true));
	}

	public void deletePlayer(UUID uniqueId) {
		this.savePlayerData(getPlayerData(uniqueId));
		this.getPlayers().remove(uniqueId);
	}

	public void sendToSpawnAndResetPlayer(Player player) {
		if (!player.isOnline()) {
			return;
		}

		PlayerData playerData = this.getPlayerData(player.getUniqueId());

		player.teleport(CustomLocation.stringToLocation(this.main.getSettingsConfig().getConfig().getString("spawn.location")).toBukkitLocation());

		playerData.setPlayerState(PlayerState.SPAWN);
	}

	public MongoCursor<Document> getPlayersSorted(String stat) {
		Document sort = new Document();
		sort.put(stat, -1);

		return this.main.getMongoManager().getPlayers().find().sort(sort).limit(10).iterator();
	}
}
