package me.trixkz.lifestealcore.managers.chatgames.chatquestion;

import lombok.AllArgsConstructor;
import lombok.Getter;
import lombok.RequiredArgsConstructor;
import lombok.Setter;
import me.trixkz.lifestealcore.LifeStealCore;
import me.trixkz.lifestealcore.managers.chatgames.ChatGame;
import me.trixkz.lifestealcore.playerdata.PlayerData;
import me.trixkz.lifestealcore.utils.Constants;
import me.trixkz.lifestealcore.utils.Utils;
import org.bukkit.entity.Player;
import org.bukkit.event.EventHandler;
import org.bukkit.event.player.AsyncPlayerChatEvent;
import org.bukkit.scheduler.BukkitRunnable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.concurrent.ThreadLocalRandom;

public class ChatQuestion extends ChatGame {

    private LifeStealCore main = LifeStealCore.getInstance();

    private List<Question> questions = new ArrayList<Question>();

    private Question question = null;

    private double time = 0;

    public ChatQuestion() {
        for (int i = 0; i < this.main.getSettingsConfig().getConfig().getConfigurationSection("CHAT-GAMES.QUESTIONS").getKeys(false).size(); i++) {
            this.questions.add(new Question(this.main.getSettingsConfig().getConfig().getString("CHAT-GAMES.QUESTIONS." + i + ".QUESTION"), this.main.getSettingsConfig().getConfig().getStringList("CHAT-GAMES.QUESTIONS." + i + ".ANSWERS")));
        }
    }

    @Override
    public String getName() {
        return Utils.translate("&d&lChat question");
    }

    @Override
    public void startGame() {
        this.setStarted(true);

        new BukkitRunnable() {
            @Override
            public void run() {
                if (!isStarted()) {
                    cancel();

                    return;
                }

                time += 0.10;
            }
        }.runTaskTimer(this.main, 5L, 5L);

        int random = ThreadLocalRandom.current().nextInt(this.questions.size());

        Question question = this.questions.get(random);

        this.question = question;

        List<String> message = new ArrayList<String>(
                Arrays.asList(
                        "",
                        this.getName(),
                        "",
                        "&7┃ &9Question: &d" + this.question.getQuestion(),
                        "&7┃ &9Respond with the right answer",
                        "&7┃ &9to get 10 thousand coins",
                        ""
                )
        );

        message.forEach((string) -> {
            this.main.getServer().broadcastMessage(Utils.translate(string));
        });

        new BukkitRunnable() {
            @Override
            public void run() {
                if (!isStarted()) {
                    cancel();

                    return;
                }

                stopGame();
            }
        }.runTaskLater(this.main, 300L);
    }

    @Override
    public void stopGame() {
        this.setStarted(false);
        this.main.getServer().broadcastMessage(Utils.translate(Constants.PREFIX + "&eNo one answered the question in time"));
    }

    @Getter
    @Setter
    @AllArgsConstructor
    @RequiredArgsConstructor
    public class Question {

        private String question;
        private List<String> answers;
    }

    @EventHandler
    public void onAsyncPlayerChat(AsyncPlayerChatEvent event) {
        Player player = event.getPlayer();

        PlayerData playerData = this.main.getPlayerDataManager().getPlayerData(player.getUniqueId());

        if (!this.isStarted()) {
            return;
        }

        if (this.question == null) {
            return;
        }

        String answer = event.getMessage();

        if (!this.question.getAnswers().contains(answer)) {
            return;
        }

        this.setStarted(false);

        event.setCancelled(true);

        List<String> message = new ArrayList<String>(
                Arrays.asList(
                        "",
                        this.getName(),
                        "",
                        "&7┃ &9Answer: &d" + answer,
                        "&7┃ &9Winner: &d" + player.getName(),
                        "&7┃ &9Time: &d" + this.time,
                        ""
                )
        );

        message.forEach((string) -> {
            this.main.getServer().broadcastMessage(Utils.translate(string));
        });

        playerData.setCoins(playerData.getCoins() + 10000);
    }
}
