package me.trixkz.lifestealcore.managers.envoys;

import lombok.Getter;
import lombok.Setter;
import me.trixkz.lifestealcore.LifeStealCore;
import me.trixkz.lifestealcore.normalmenusystem.ItemStackButton;
import me.trixkz.lifestealcore.utils.CustomLocation;
import org.bukkit.Material;
import org.bukkit.configuration.ConfigurationSection;
import org.bukkit.configuration.file.FileConfiguration;
import org.bukkit.enchantments.Enchantment;
import org.bukkit.inventory.ItemStack;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.atomic.AtomicInteger;

@Getter
@Setter
public class EnvoyManager {

    private LifeStealCore main = LifeStealCore.getInstance();

    private List<Envoy> envoys = new ArrayList<Envoy>();
    private List<CustomLocation> envoyLocations = new ArrayList<CustomLocation>();

    private boolean started = false;

    public EnvoyManager() {
        FileConfiguration fileConfiguration = this.main.getEnvoysConfig().getConfig();

        ConfigurationSection configurationSection = fileConfiguration.getConfigurationSection("envoys");

        AtomicInteger atomicInteger = new AtomicInteger(0);

        if (configurationSection == null) {
            return;
        }

        configurationSection.getKeys(false).forEach((number) -> {
            List<ItemStack> itemStacks = new ArrayList<ItemStack>();
            List<String> commands = new ArrayList<String>();

            double chance = 0;

            String name = configurationSection.getString(number + ".item-stacks." + atomicInteger + ".name");

            List<String> loreArrayList = configurationSection.getStringList(number + ".item-stacks." + atomicInteger + ".lore");

            String[] lore = new String[loreArrayList.size()];

            for (int i = 0; i < loreArrayList.size(); i++) {
                lore[i] = loreArrayList.get(i);
            }

            Material material = Material.getMaterial(configurationSection.getString(number + ".item-stacks." + atomicInteger + ".material"));

            int data = configurationSection.getInt(number + ".item-stacks." + atomicInteger + ".data");
            int amount = configurationSection.getInt(number + ".item-stacks." + atomicInteger + ".amount");

            Map<Enchantment, Integer> enchantments = new HashMap<Enchantment, Integer>();

            if (configurationSection.getStringList(number + ".item-stacks." + atomicInteger + ".enchantments") != null) {
                configurationSection.getStringList(number + ".item-stacks." + atomicInteger + ".enchantments").forEach((string) -> {
                    String[] stringArray = string.split(":");

                    enchantments.put(Enchantment.getByName(stringArray[0]), Integer.parseInt(stringArray[1]));
                });
            }

            ItemStackButton itemStackButton = new ItemStackButton(name, lore, material, data, amount);

            ItemStack itemStack = itemStackButton.makeItemStack();

            if (enchantments.size() >= 1) {
                itemStack.addEnchantments(enchantments);
            }

            itemStacks.add(itemStack);

            chance = configurationSection.getDouble(number + ".chance");

            this.envoys.add(new Envoy(new EnvoyReward(itemStacks, chance)));

            atomicInteger.addAndGet(1);
        });

        this.loadEnvoyLocations();
    }

    public void loadEnvoyLocations() {
        this.main.getEnvoysConfig().getConfig().getStringList("envoy-locations").forEach((envoyLocation) -> {
            this.envoyLocations.add(CustomLocation.stringToLocation(envoyLocation));
        });
    }

    public void saveEnvoyLocations() {
        List<String> envoyLocations = new ArrayList<String>();

        this.envoyLocations.forEach((envoyLocation) -> {
            envoyLocations.add(CustomLocation.locationToString(envoyLocation));
        });

        this.main.getEnvoysConfig().getConfig().set("envoy-locations", envoyLocations);
        this.main.getEnvoysConfig().save();
    }

    public void startEnvoyEvent() {
        this.started = true;
    }
}
