package me.trixkz.lifestealcore.managers.event;

import com.google.common.primitives.Ints;
import lombok.AllArgsConstructor;
import lombok.Getter;
import lombok.RequiredArgsConstructor;
import lombok.Setter;

import java.util.Calendar;
import java.util.Date;
import java.util.Objects;

@Getter
@Setter
@AllArgsConstructor
@RequiredArgsConstructor
public class EventScheduledTime {

    private int day;
    private int hour;
    private int minutes;

    public static EventScheduledTime parse(String input) {
        String[] inputSplit = input.split(" ");

        int days = Integer.parseInt(inputSplit[0]);

        String[] timeSplit = inputSplit[1].split(":");
        int hour = Integer.parseInt(timeSplit[0]);
        int minutes = timeSplit.length > 1 ? Integer.parseInt(timeSplit[1]) : 0;

        return (new EventScheduledTime(days, hour, minutes));
    }

    public static EventScheduledTime parse(Date input) {
        Calendar calendar = Calendar.getInstance();

        calendar.setTime(input);

        int days = calendar.get(Calendar.DAY_OF_YEAR);
        int hour = calendar.get(Calendar.HOUR_OF_DAY);
        int minutes = calendar.get(Calendar.MINUTE);

        return (new EventScheduledTime(days, hour, minutes));
    }

    public Date toDate() {
        Calendar activationTime = Calendar.getInstance();

        activationTime.set(Calendar.DAY_OF_YEAR, day);
        activationTime.set(Calendar.HOUR_OF_DAY, hour);
        activationTime.set(Calendar.MINUTE, minutes);
        activationTime.set(Calendar.SECOND, 0);
        activationTime.set(Calendar.MILLISECOND, 0);

        return (activationTime.getTime());
    }


    @Override
    public boolean equals(Object object) {
        if (object instanceof EventScheduledTime) {
            EventScheduledTime other = (EventScheduledTime) object;

            return (other.day == this.day && other.hour == this.hour && other.minutes == this.minutes);
        }

        return (false);
    }


    @Override
    public int hashCode() {
        return Objects.hash(day, hour, minutes);
    }

    public int compare(EventScheduledTime other) {
        int result = Ints.compare(day, other.day);

        if (result == 0) {
            result = Ints.compare(hour, other.hour);
        }

        if (result == 0) {
            result = Ints.compare(minutes, other.minutes);
        }

        return (result);
    }

    @Override
    public String toString() {
        return (day + " " + hour + ":" + minutes);
    }
}
