package me.trixkz.lifestealcore.managers.koth;

import lombok.Getter;
import lombok.Setter;
import me.trixkz.lifestealcore.LifeStealCore;
import me.trixkz.lifestealcore.events.*;
import me.trixkz.lifestealcore.utils.CustomLocation;
import me.trixkz.lifestealcore.utils.Utils;
import org.bukkit.GameMode;
import org.bukkit.entity.Player;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.UUID;

@Getter
@Setter
public class KOTH {

    private LifeStealCore main = LifeStealCore.getInstance();

    private String name;
    private String world;
    private CustomLocation customLocation;
    private int time;
    private int distance;
    private boolean active = false;
    private boolean hidden = false;

    private UUID player;
    private int remainingTime;
    private boolean stop;

    public KOTH(String name, CustomLocation customLocation) {
        this.name = name;
        this.world = customLocation.getWorld();
        this.customLocation = customLocation;
        this.time = 900;
        this.distance = 3;
        this.stop = false;
        this.main.getKothManager().makeKOTH(this.name, this);
        this.main.getKothManager().saveKOTHs();
    }

    public KOTH(String name, String world, CustomLocation customLocation, int time, int distance, boolean active, boolean hidden) {
        this.name = name;
        this.world = world;
        this.customLocation = customLocation;
        this.time = time;
        this.distance = distance;
        this.active = active;
        this.hidden = hidden;
    }

    public boolean enable() {
        if (this.active) {
            return false;
        }

        KOTHEnabledEvent eventEnabledEvent = new KOTHEnabledEvent(this);

        this.main.getServer().getPluginManager().callEvent(eventEnabledEvent);
        this.active = true;
        this.player = null;
        this.remainingTime = this.time;
        this.stop = false;

        return true;
    }

    public boolean disable() {
        if (!this.active) {
            return false;
        }

        KOTHDisabledEvent eventDisabledEvent = new KOTHDisabledEvent(this);

        this.main.getServer().getPluginManager().callEvent(eventDisabledEvent);
        this.active = false;
        this.player = null;
        this.remainingTime = this.time;
        this.stop = false;

        return true;
    }

    public void setActive(boolean active) {
        this.active = active;
    }

    public boolean isActive() {
        return this.active;
    }

    public void setHidden(boolean hidden) {
        this.hidden = hidden;
        this.main.getKothManager().saveKOTHs();
    }

    public boolean isHidden() {
        return this.hidden;
    }

    public void tick() {
        if (this.player != null) {
            Player player = this.main.getServer().getPlayer(this.player);

            if (player == null || !this.isInCapArea(player) || player.isDead() || player.getGameMode() != GameMode.SURVIVAL || player.hasMetadata("Invisible")) {
                this.restartTime();
            } else {
                if (this.remainingTime > 1 && this.remainingTime % 10 == 0 && !this.isHidden()) {
                    player.sendMessage(Utils.translate("&8[&9KOTH&8] &eYou are trying to cap &d" + this.name));
                }

                if (this.remainingTime > 1 && this.remainingTime % 60 == 0 && !this.isHidden()) {
                    for (Player playerOnline : this.main.getServer().getOnlinePlayers()) {
                        if (this.main.getClanManager().isAClanMember(playerOnline) && player != playerOnline) {
                            playerOnline.sendMessage(Utils.translate("&8[&9KOTH&8] &eYour island is capping &d" + this.name));
                        }
                    }
                }

                if (this.remainingTime <= 0) {
                    this.stopCapping();
                } else {
                    KOTHCapTickEvent eventCapTickEvent = new KOTHCapTickEvent(this);

                    this.main.getServer().getPluginManager().callEvent(eventCapTickEvent);
                }

                this.remainingTime--;
            }
        } else {
            List<Player> players = new ArrayList<Player>();

            for (Player player : this.main.getServer().getOnlinePlayers()) {
                if (this.isInCapArea(player) && !player.isDead() && player.getGameMode() == GameMode.SURVIVAL && !player.hasMetadata("Invisible")) {
                    players.add(player);
                }
            }

            Collections.shuffle(players);

            if (players.size() != 0) {
                this.startCapping(players.get(0));
            }
        }
    }

    public void setLocation(CustomLocation customLocation) {
        this.customLocation = customLocation;
        this.world = customLocation.getWorld();
        this.main.getKothManager().saveKOTHs();
    }

    public void setTime(int time) {
        int oldTime = this.time;

        this.time = time;

        if (this.remainingTime > time) {
            this.remainingTime = time;
        } else if (this.remainingTime == oldTime) {
            this.remainingTime = time;
        }

        this.main.getKothManager().saveKOTHs();
    }

    public void setDistance(int distance) {
        this.distance = distance;
        this.main.getKothManager().saveKOTHs();
    }

    public void startCapping(Player player) {
        if (this.player != null) {
            this.restartTime();
        }

        this.player = player.getUniqueId();
        this.remainingTime = this.time;
    }

    public boolean stopCapping() {
        Player player = this.main.getServer().getPlayer(this.player);

        if (player == null) {
            this.restartTime();

            return false;
        }

        KOTHCappedEvent eventCappedEvent = new KOTHCappedEvent(player, this);

        this.main.getServer().getPluginManager().callEvent(eventCappedEvent);

        if (eventCappedEvent.isCancelled()) {
            this.restartTime();

            return false;
        }

        this.disable();

        return true;
    }

    public void restartTime() {
        KOTHCapLostEvent kothCapLostEvent = new KOTHCapLostEvent(this);

        this.main.getServer().getPluginManager().callEvent(kothCapLostEvent);
        this.player = null;
        this.remainingTime = this.time;

        if (this.stop) {
            this.disable();
            this.main.getServer().broadcastMessage(Utils.translate("&8[&9KOTH&8] &d" + this.name + " &ehas been stopped"));
        }
    }

    public boolean isInCapArea(Player player) {
        if (!player.getLocation().getWorld().getName().equalsIgnoreCase(this.world)) {
            return false;
        }

        int x = Math.abs(player.getLocation().getBlockX() - this.customLocation.toBukkitLocation().getBlockX());
        int y = Math.abs(player.getLocation().getBlockY() - this.customLocation.toBukkitLocation().getBlockY());
        int z = Math.abs(player.getLocation().getBlockZ() - this.customLocation.toBukkitLocation().getBlockZ());

        return x <= this.distance && y <= 5 && z <= this.distance;
    }
}
