package me.trixkz.lifestealcore.managers.pvpclass;

import lombok.Getter;
import lombok.Setter;
import me.trixkz.lifestealcore.LifeStealCore;
import me.trixkz.lifestealcore.playerdata.PlayerData;
import me.trixkz.lifestealcore.playerdata.PlayerState;
import me.trixkz.lifestealcore.utils.Constants;
import me.trixkz.lifestealcore.utils.Pair;
import me.trixkz.lifestealcore.utils.TimeUtils;
import me.trixkz.lifestealcore.utils.Utils;
import org.bukkit.Location;
import org.bukkit.Material;
import org.bukkit.entity.Arrow;
import org.bukkit.entity.Player;
import org.bukkit.entity.Projectile;
import org.bukkit.event.EventHandler;
import org.bukkit.event.entity.EntityDamageByEntityEvent;
import org.bukkit.event.entity.EntityShootBowEvent;
import org.bukkit.metadata.FixedMetadataValue;
import org.bukkit.potion.PotionEffect;
import org.bukkit.potion.PotionEffectType;
import java.util.*;
import java.util.concurrent.TimeUnit;

@Getter
@Setter
public class ArcherClass extends PvPClass {

    private LifeStealCore main = LifeStealCore.getInstance();

    private Map<UUID, Long> taggedPlayers = new HashMap<UUID, Long>();
    private Map<UUID, Set<Pair<UUID, Long>>> taggedByPlayers = new HashMap<UUID, Set<Pair<UUID, Long>>>();
    private Map<UUID, Long> lastSpeedUsage = new HashMap<UUID, Long>();
    private Map<UUID, Long> lastJumpUsage = new HashMap<UUID, Long>();

    private int tagSeconds = 7;

    public ArcherClass() {
        super("Archer", 5, Arrays.asList(Material.SUGAR, Material.FEATHER));
    }

    @Override
    public void apply(Player player) {
        player.addPotionEffect(new PotionEffect(PotionEffectType.SPEED, Integer.MAX_VALUE, 2));
        player.addPotionEffect(new PotionEffect(PotionEffectType.DAMAGE_RESISTANCE, Integer.MAX_VALUE, 1));
    }

    @Override
    public void tick(Player player) {
        if (!player.hasPotionEffect(PotionEffectType.SPEED)) {
            player.addPotionEffect(new PotionEffect(PotionEffectType.SPEED, Integer.MAX_VALUE, 2));
        }

        if (!player.hasPotionEffect(PotionEffectType.DAMAGE_RESISTANCE)) {
            player.addPotionEffect(new PotionEffect(PotionEffectType.DAMAGE_RESISTANCE, Integer.MAX_VALUE, 1));
        }
    }

    @Override
    public void remove(Player player) {

    }

    @Override
    public boolean isItemConsumed(Player player, Material material) {
        PlayerData playerData = this.main.getPlayerDataManager().getPlayerData(player.getUniqueId());

        if (playerData.getPlayerState() != PlayerState.WARZONE) {
            player.sendMessage(Utils.translate(Constants.PREFIX + "You can only use this in the warzone"));

            return false;
        }

        if (material == Material.SUGAR) {
            if (this.lastSpeedUsage.containsKey(player.getUniqueId())) {
                if (this.lastSpeedUsage.get(player.getUniqueId()) > System.currentTimeMillis()) {
                    long amountOfTimeLeft = this.lastSpeedUsage.get(player.getUniqueId()) - System.currentTimeMillis();

                    player.sendMessage(Utils.translate(Constants.PREFIX + "You can not use this for another " + TimeUtils.formatIntoDetailedString((int) amountOfTimeLeft / 1000)));

                    return false;
                }
            }

            this.lastSpeedUsage.put(player.getUniqueId(), System.currentTimeMillis() + TimeUnit.SECONDS.toMillis(30));

            player.addPotionEffect(new PotionEffect(PotionEffectType.SPEED, 200, 3));

            return true;
        } else if (material == Material.FEATHER) {
            if (this.lastJumpUsage.containsKey(player.getUniqueId())) {
                if (this.lastJumpUsage.get(player.getUniqueId()) > System.currentTimeMillis()) {
                    long amountOfTimeLeft = this.lastJumpUsage.get(player.getUniqueId()) - System.currentTimeMillis();

                    player.sendMessage(Utils.translate(Constants.PREFIX + "You can not use this for another " + TimeUtils.formatIntoDetailedString((int) amountOfTimeLeft / 1000)));

                    return false;
                }
            }

            this.lastSpeedUsage.put(player.getUniqueId(), System.currentTimeMillis() + TimeUnit.SECONDS.toMillis(30));

            player.addPotionEffect(new PotionEffect(PotionEffectType.JUMP, 100, 4));

            // to do - make a spawn tag manager

            return true;
        }

        return false;
    }

    @Override
    public boolean qualifies(Player player) {
        return this.hasAllArmor(player) && player.getInventory().getHelmet().getType() == Material.LEATHER_HELMET && player.getInventory().getChestplate().getType() == Material.LEATHER_CHESTPLATE && player.getInventory().getLeggings().getType() == Material.LEATHER_LEGGINGS && player.getInventory().getBoots().getType() == Material.LEATHER_BOOTS;
    }

    @EventHandler
    public void onEntityDamageByEntityOne(EntityDamageByEntityEvent event) {
        if (event.getEntity() instanceof Player) {
            Player player = (Player) event.getEntity();

            if (this.isTagged(player)) {
                Player damager = null;
                if (event.getDamager() instanceof Player) {
                    damager = (Player) event.getDamager();
                } else if (event.getDamager() instanceof Projectile && ((Projectile) event.getDamager()).getShooter() instanceof Player) {
                    damager = (Player) ((Projectile) event.getDamager()).getShooter();
                }

                if (damager != null && !this.canUseTag(damager, player)) {
                    return;
                }

                event.setDamage(event.getDamage() * 1.25D);
            }
        }
    }

    @EventHandler
    public void onEntityDamageByEntityTwo(EntityDamageByEntityEvent event) {
        if (event.getEntity() instanceof Player && event.getDamager() instanceof Arrow) {
            Arrow arrow = (Arrow) event.getDamager();

            Player victim = (Player) event.getEntity();

            if (!(arrow.getShooter() instanceof Player)) {
                return;
            }

            Player shooter = (Player) arrow.getShooter();

            float pullback = arrow.getMetadata("Archer bow pullback").get(0).asFloat();

            if (!this.main.getPvpClassManager().hasPvPClassOn(shooter, this)) {
                return;
            }

            int damage = this.isTagged(victim) ? 4 : 3;

            if (pullback < 0.5F) {
                damage = 2;
            }

            if (victim.getHealth() - damage <= 0D) {
                event.setCancelled(true);
            } else {
                event.setDamage(0D);
            }

            Location shotFrom = (Location) arrow.getMetadata("Archer bow shot from distance").get(0).value();

            double distance = shotFrom.distance(victim.getLocation());

            victim.setHealth(Math.max(0D, victim.getHealth() - damage));

            if (this.main.getPvpClassManager().hasPvPClassOn(victim, this)) {
                shooter.sendMessage(Utils.translate("&8[&9Arrow Range &8(&d" + ((int) distance) + "&8)] &eCan not tag other archers &8(&d" + (damage / 2) + " heart" + (((damage / 2) == 1) ? "" : "s") + "&8)"));
            } else if (pullback >= 0.5F) {
                shooter.sendMessage(Utils.translate("&8[&9Arrow Range &8(&d" + ((int) distance) + "&8)] &eTagged player for " + this.tagSeconds + " seconds &8(&d" + (damage / 2) + " heart" + (((damage / 2) == 1) ? "" : "s") + "&8)"));

                if (!this.isTagged(victim)) {
                    victim.sendMessage(Utils.translate("&8[&9Tagged&8] &eAn archer has shot you and tagged you (+25% damage) for " + this.tagSeconds + " seconds"));
                }

                this.taggedPlayers.put(victim.getUniqueId(), System.currentTimeMillis() + (this.tagSeconds * 1000));

                this.taggedByPlayers.putIfAbsent(shooter.getUniqueId(), new HashSet<>());
                this.taggedByPlayers.get(shooter.getUniqueId()).add(new Pair<UUID, Long>(victim.getUniqueId(), System.currentTimeMillis() + (this.tagSeconds * 1000)));
            } else {
                shooter.sendMessage(Utils.translate("&8[&9Arrow Range &8(&d" + ((int) distance) + "&8)] &eBow was not fully pulled back &8(&d" + (damage / 2) + " heart" + (((damage / 2) == 1) ? "" : "s") + "&8)"));
            }
        }
    }

    @EventHandler
    public void onEntityShootBow(EntityShootBowEvent event) {
        event.getProjectile().setMetadata("Archer bow pullback", new FixedMetadataValue(this.main, event.getForce()));
        event.getProjectile().setMetadata("Archer bow shot from distance", new FixedMetadataValue(this.main, event.getProjectile().getLocation()));
    }

    public boolean canUseTag(Player player, Player target) {
        if (this.taggedByPlayers.containsKey(player.getUniqueId())) {
            for (Pair<UUID, Long> taggedByPlayer : this.taggedByPlayers.get(player.getUniqueId())) {
                if (target.getUniqueId().equals(taggedByPlayer.getFirstPair()) && taggedByPlayer.getSecondPair() > System.currentTimeMillis()) {
                    return false;
                }
            }
        }

        return true;
    }

    public boolean isTagged(Player player) {
        if (!this.taggedPlayers.containsKey(player.getUniqueId())) {
            return false;
        }

        return this.taggedPlayers.get(player.getUniqueId()) > System.currentTimeMillis();
    }
}
