package me.trixkz.lifestealcore.managers.pvpclass;

import lombok.Getter;
import lombok.Setter;
import me.trixkz.lifestealcore.LifeStealCore;
import me.trixkz.lifestealcore.managers.clans.Clan;
import me.trixkz.lifestealcore.playerdata.PlayerData;
import me.trixkz.lifestealcore.playerdata.PlayerState;
import me.trixkz.lifestealcore.utils.Constants;
import me.trixkz.lifestealcore.utils.Utils;
import org.bukkit.GameMode;
import org.bukkit.Material;
import org.bukkit.entity.Entity;
import org.bukkit.entity.Player;
import org.bukkit.event.EventHandler;
import org.bukkit.event.player.PlayerInteractEvent;
import org.bukkit.inventory.ItemStack;
import org.bukkit.potion.PotionEffect;
import org.bukkit.potion.PotionEffectType;
import org.bukkit.scheduler.BukkitRunnable;
import java.util.*;

@Getter
@Setter
public class BardClass extends PvPClass {

    private LifeStealCore main = LifeStealCore.getInstance();

    private Map<Material, BardEffect> clickEffects = new HashMap<Material, BardEffect>();
    private Map<Material, BardEffect> passiveEffects = new HashMap<Material, BardEffect>();
    private Map<UUID, Float> energy = new HashMap<UUID, Float>();
    private Map<UUID, Long> lastEffectUsage = new HashMap<UUID, Long>();

    private int range = 40;
    private int effectCooldown = 10000;
    private float maximumEnergy = 120;
    private float energyRegenerationPerSecond = 1;

    public BardClass() {
        super("Bard", 5, null);

        this.clickEffects.put(Material.BLAZE_POWDER, new BardEffect(new PotionEffect(PotionEffectType.INCREASE_DAMAGE, 100, 1), 45));
        this.clickEffects.put(Material.SUGAR, new BardEffect(new PotionEffect(PotionEffectType.SPEED, 120, 2), 20));
        this.clickEffects.put(Material.FEATHER, new BardEffect(new PotionEffect(PotionEffectType.JUMP, 100, 6), 25));
        this.clickEffects.put(Material.IRON_INGOT, new BardEffect(new PotionEffect(PotionEffectType.DAMAGE_RESISTANCE, 100, 2), 40));
        this.clickEffects.put(Material.GHAST_TEAR, new BardEffect(new PotionEffect(PotionEffectType.REGENERATION, 100, 2), 40));
        this.clickEffects.put(Material.MAGMA_CREAM, new BardEffect(new PotionEffect(PotionEffectType.FIRE_RESISTANCE, 900, 0), 40));
        this.clickEffects.put(Material.INK_SAC, new BardEffect(new PotionEffect(PotionEffectType.INVISIBILITY, 900, 1), 35));
        this.clickEffects.put(Material.SPIDER_EYE, new BardEffect(new PotionEffect(PotionEffectType.WITHER, 100, 1), 35));
        this.clickEffects.put(Material.WHEAT, new BardEffect(null, 25));
        this.passiveEffects.put(Material.BLAZE_POWDER, new BardEffect(new PotionEffect(PotionEffectType.INCREASE_DAMAGE, 100, 1), 45));
        this.passiveEffects.put(Material.SUGAR, new BardEffect(new PotionEffect(PotionEffectType.SPEED, 120, 2), 20));
        this.passiveEffects.put(Material.FEATHER, new BardEffect(new PotionEffect(PotionEffectType.JUMP, 100, 6), 25));
        this.passiveEffects.put(Material.IRON_INGOT, new BardEffect(new PotionEffect(PotionEffectType.DAMAGE_RESISTANCE, 100, 2), 40));
        this.passiveEffects.put(Material.GHAST_TEAR, new BardEffect(new PotionEffect(PotionEffectType.REGENERATION, 100, 2), 40));
        this.passiveEffects.put(Material.MAGMA_CREAM, new BardEffect(new PotionEffect(PotionEffectType.FIRE_RESISTANCE, 900, 0), 40));
        this.passiveEffects.put(Material.INK_SAC, new BardEffect(new PotionEffect(PotionEffectType.INVISIBILITY, 900, 1), 35));

        new BukkitRunnable() {
            @Override
            public void run() {
                for (Player player : main.getServer().getOnlinePlayers()) {
                    PlayerData playerData = main.getPlayerDataManager().getPlayerData(player.getUniqueId());

                    if (!main.getPvpClassManager().hasPvPClassOn(player, BardClass.this) || playerData.getPlayerState() != PlayerState.WARZONE) {
                        continue;
                    }

                    if (energy.containsKey(player.getUniqueId())) {
                        if (energy.get(player.getUniqueId()) == maximumEnergy) {
                            continue;
                        }

                        energy.put(player.getUniqueId(), Math.min(maximumEnergy, energy.get(player.getUniqueId()) + energyRegenerationPerSecond));
                    } else {
                        energy.put(player.getUniqueId(), 0F);
                    }

                    int energyNumber = energy.get(player.getUniqueId()).intValue();

                    if (energyNumber % 10 == 0) {
                        player.sendMessage(Utils.translate("&8[&9Bard&8] &eBard energy: " + energyNumber));
                    }
                }
            }
        }.runTaskTimer(this.main, 15L, 20L);
    }

    @Override
    public void apply(Player player) {
        PlayerData playerData = this.main.getPlayerDataManager().getPlayerData(player.getUniqueId());

        player.addPotionEffect(new PotionEffect(PotionEffectType.SPEED, Integer.MAX_VALUE, 1));
        player.addPotionEffect(new PotionEffect(PotionEffectType.DAMAGE_RESISTANCE, Integer.MAX_VALUE, 1));
        player.addPotionEffect(new PotionEffect(PotionEffectType.REGENERATION, Integer.MAX_VALUE, 0));

        if (playerData.getPlayerState() != PlayerState.WARZONE) {
            player.sendMessage(Utils.translate(Constants.PREFIX + "You can only use this in the warzone"));
        }
    }

    @Override
    public void tick(Player player) {
        PlayerData playerData = this.main.getPlayerDataManager().getPlayerData(player.getUniqueId());

        if (!player.hasPotionEffect(PotionEffectType.SPEED)) {
            player.addPotionEffect(new PotionEffect(PotionEffectType.SPEED, Integer.MAX_VALUE, 1));
        }

        if (!player.hasPotionEffect(PotionEffectType.DAMAGE_RESISTANCE)) {
            player.addPotionEffect(new PotionEffect(PotionEffectType.DAMAGE_RESISTANCE, Integer.MAX_VALUE, 1));
        }

        if (!player.hasPotionEffect(PotionEffectType.REGENERATION)) {
            player.addPotionEffect(new PotionEffect(PotionEffectType.REGENERATION, Integer.MAX_VALUE, 0));
        }

        if (player.getItemInHand() != null) {
            if (this.passiveEffects.containsKey(player.getItemInHand().getType()) && playerData.getPlayerState() == PlayerState.WARZONE) {
                this.giveBardEffect(player, this.passiveEffects.get(player.getItemInHand().getType()));
            }
        }
    }

    @Override
    public void remove(Player player) {
        this.energy.remove(player.getUniqueId());

        for (BardEffect bardEffect : this.clickEffects.values()) {
            bardEffect.getLastMessageSent().remove(player.getUniqueId());
        }

        for (BardEffect bardEffect : this.passiveEffects.values()) {
            bardEffect.getLastMessageSent().remove(player.getUniqueId());
        }
    }

    @Override
    public boolean isItemConsumed(Player player, Material material) {
        return true;
    }

    @Override
    public boolean qualifies(Player player) {
        return this.hasAllArmor(player) && player.getInventory().getHelmet().getType() == Material.GOLDEN_HELMET && player.getInventory().getChestplate().getType() == Material.GOLDEN_CHESTPLATE && player.getInventory().getLeggings().getType() == Material.GOLDEN_LEGGINGS && player.getInventory().getBoots().getType() == Material.GOLDEN_BOOTS;
    }

    @EventHandler
    public void onPlayerInteract(PlayerInteractEvent event) {
        Player player = event.getPlayer();

        PlayerData playerData = this.main.getPlayerDataManager().getPlayerData(player.getUniqueId());

        if (!event.hasItem() || !event.getAction().name().contains("RIGHT_") || !this.clickEffects.containsKey(event.getItem().getType()) || !this.main.getPvpClassManager().hasPvPClassOn(player, this) || !energy.containsKey(player.getUniqueId())) {
            return;
        }

        if (playerData.getPlayerState() != PlayerState.WARZONE) {
            player.sendMessage(Utils.translate(Constants.PREFIX + "You can only use this in the warzone"));

            return;
        }

        if (this.lastEffectUsage.containsKey(player.getUniqueId())) {
            if (this.lastEffectUsage.get(player.getUniqueId()) > System.currentTimeMillis() && player.getGameMode() != GameMode.CREATIVE) {
                long amountOfTimeLeft = this.lastEffectUsage.get(player.getUniqueId()) - System.currentTimeMillis();

                double value = amountOfTimeLeft / 1000;
                double amountOfTimeInSeconds = Math.round(10 * value) / 10;

                player.sendMessage(Utils.translate("&8[&9Bard&8] &eYou can not use this for another " + amountOfTimeInSeconds + " seconds"));

                return;
            }
        }

        BardEffect bardEffect = this.clickEffects.get(event.getItem().getType());

        if (bardEffect.getEnergy() > this.energy.get(player.getUniqueId())) {
            player.sendMessage(Utils.translate("&8[&9Bard&8] &eYou do not have enough energy for this, you need " + bardEffect.getEnergy() + " energy, but you have " + this.energy.get(player.getUniqueId()).intValue() + " energy"));

            return;
        }

        String string = "";

        int amountOfPlayers = this.getNearbyPlayers(player).size();

        switch (event.getItem().getType()) {
            case BLAZE_POWDER:
                string = "&4Strength 2";

                break;
            case SUGAR:
                string = "&bSpeed 3";

                break;
            case FEATHER:
                string = "&fJump Boost 7";

                break;
            case IRON_INGOT:
                string = "&9Resistance 3";

                break;
            case GHAST_TEAR:
                string = "&dRegeneration 3";

                break;
            case MAGMA_CREAM:
                string = "&cFire Resistance 1";

                break;
            case INK_SAC:
                string = "&fInvisibility 2";

                break;
            case SPIDER_EYE:
                string = "&0Wither 1";

                break;
        }

        if (amountOfPlayers == 1) {
            player.sendMessage(Utils.translate("&8[&9Bard&8] &eYou have given " + string + " &eto yourself"));
        } else {
            player.sendMessage(Utils.translate("&8[&9Bard&8] &eYou have given " + string + " &eto " + amountOfPlayers + " players"));
        }

        this.giveBardEffect(player, bardEffect);
        this.energy.put(player.getUniqueId(), this.energy.get(player.getUniqueId()) - bardEffect.getEnergy());
        this.lastEffectUsage.put(player.getUniqueId(), System.currentTimeMillis() + this.effectCooldown);

        if (player.getItemInHand().getAmount() == 1) {
            player.setItemInHand(new ItemStack(Material.AIR));
            player.updateInventory();
        } else {
            player.getItemInHand().setAmount(player.getItemInHand().getAmount() - 1);
        }
    }

    public void giveBardEffect(Player player, BardEffect bardEffect) {
        PlayerData playerData = this.main.getPlayerDataManager().getPlayerData(player.getUniqueId());

        for (Player playerNearPlayer : this.getNearbyPlayers(player)) {
            if (playerData.getPlayerState() != PlayerState.WARZONE) {
                continue;
            }

            if (this.main.getPvpClassManager().hasPvPClassOn(playerNearPlayer, this)) {
                if (bardEffect.getPotionEffect() != null) {
                    if (bardEffect.getPotionEffect().getType().equals(PotionEffectType.INCREASE_DAMAGE)) {
                        continue;
                    }
                }
            }

            if (bardEffect.getPotionEffect() != null) {
                for (PotionEffect potionEffect : playerNearPlayer.getActivePotionEffects()) {
                    new BukkitRunnable() {
                        @Override
                        public void run() {
                            playerNearPlayer.addPotionEffect(potionEffect);
                        }
                    }.runTaskLater(this.main, bardEffect.getPotionEffect().getDuration() + 2);
                }

                this.addPotionEffect(player, bardEffect.getPotionEffect());
            } else {
                Material material = player.getItemInHand().getType();

                this.giveBardEffect(playerNearPlayer, material);
            }
        }
    }

    public void giveBardEffect(Player player, Material material) {
        switch (material) {
            case FERMENTED_SPIDER_EYE:
                break;
        }
    }

    public List<Player> getNearbyPlayers(Player player) {
        List<Player> players = new ArrayList<Player>();
        players.add(player);

        PlayerData playerData = this.main.getPlayerDataManager().getPlayerData(player.getUniqueId());

        if (playerData.getClanOwner() == null) {
            return players;
        }

        Clan clan = this.main.getClanManager().getClan(playerData.getClanOwner());

        if (clan == null) {
            return players;
        }

        for (Entity entity : player.getNearbyEntities(this.range, this.range / 2, this.range)) {
            if (entity instanceof Player) {
                Player playerNearPlayer = (Player) entity;

                PlayerData playerNearPlayerPlayerData = this.main.getPlayerDataManager().getPlayerData(playerNearPlayer.getUniqueId());

                if (playerNearPlayerPlayerData.getPlayerState() != PlayerState.WARZONE) {
                    continue;
                }

                if (this.main.getClanManager().isAClanMember(playerNearPlayer)) {
                    players.add(playerNearPlayer);
                }
            }
        }

        return players;
    }
}
