package me.trixkz.lifestealcore.managers.pvpclass;

import com.google.common.collect.HashBasedTable;
import com.google.common.collect.Table;
import lombok.Getter;
import lombok.Setter;
import org.bukkit.Material;
import org.bukkit.entity.Player;
import org.bukkit.event.Listener;
import org.bukkit.potion.PotionEffect;
import org.bukkit.potion.PotionEffectType;

import java.util.Collection;
import java.util.List;
import java.util.UUID;

@Getter
@Setter
public abstract class PvPClass implements Listener {

    private String name;
    private int warmupTime;
    private List<Material> consumables;

    private Table<UUID, PotionEffectType, PotionEffect> restores = HashBasedTable.create();

    public PvPClass(String name, int warmupTime, List<Material> consumables) {
        this.name = name;
        this.warmupTime = warmupTime;
        this.consumables = consumables;
    }

    public void apply(Player player) {

    }

    public void tick(Player player) {

    }

    public void remove(Player player) {

    }

    public boolean canApply(Player player) {
        return true;
    }

    public boolean isItemConsumed(Player player, Material material) {
        return true;
    }

    public abstract boolean qualifies(Player player);

    protected boolean hasAllArmor(Player player) {
        return player.getInventory().getHelmet() != null && player.getInventory().getChestplate() != null && player.getInventory().getLeggings() != null && player.getInventory().getBoots() != null;
    }

    public void addPotionEffect(Player player, PotionEffect potionEffect) {
        this.setRestorePotionEffect(player, potionEffect);
    }

    public void setRestorePotionEffect(Player player, PotionEffect potionEffect) {
        boolean cancel = true;

        Collection<PotionEffect> activePotionEffects = player.getActivePotionEffects();

        for (PotionEffect activePotionEffect : activePotionEffects) {
            if (!potionEffect.getType().equals(activePotionEffect.getType())) {
                continue;
            }

            if (potionEffect.getAmplifier() < activePotionEffect.getAmplifier()) {
                return;
            } else if (potionEffect.getAmplifier() == activePotionEffect.getAmplifier()) {
                if ((potionEffect.getDuration() <= activePotionEffect.getDuration() || potionEffect.getDuration() - activePotionEffect.getDuration() < 10) && activePotionEffect.getDuration() > 0) {
                    return;
                }
            }

            this.restores.put(player.getUniqueId(), activePotionEffect.getType(), activePotionEffect);

            cancel = false;

            break;
        }

        player.addPotionEffect(potionEffect, true);

        if (cancel && potionEffect.getDuration() > 120 && potionEffect.getDuration() < 9600) {
            this.restores.remove(player.getUniqueId(), potionEffect.getType());
        }
    }

    public void removeInfiniteEffects(Player player) {
        for (PotionEffect potionEffect : player.getActivePotionEffects()) {
            if (potionEffect.getDuration() > 1_000_000) {
                player.removePotionEffect(potionEffect.getType());
            }
        }
    }
}
