package me.trixkz.lifestealcore.managers.pvpclass;

import lombok.Getter;
import lombok.Setter;
import me.trixkz.lifestealcore.LifeStealCore;
import me.trixkz.lifestealcore.utils.Constants;
import me.trixkz.lifestealcore.utils.Utils;
import org.bukkit.entity.Player;
import org.bukkit.event.EventHandler;
import org.bukkit.event.Listener;
import org.bukkit.event.block.Action;
import org.bukkit.event.player.PlayerInteractEvent;
import org.bukkit.event.player.PlayerJoinEvent;
import org.bukkit.event.player.PlayerKickEvent;
import org.bukkit.event.player.PlayerQuitEvent;
import org.bukkit.potion.PotionEffect;

import java.util.*;

@Getter
@Setter
public class PvPClassManager implements Listener {

    private LifeStealCore main = LifeStealCore.getInstance();

    private List<PvPClass> pvpClasses = new ArrayList<PvPClass>();
    private Map<UUID, PvPClass> equippedPvPClasses = new HashMap<UUID, PvPClass>();
    private Map<UUID, SavedPotionEffect> savedPotionEffects = new HashMap<UUID, SavedPotionEffect>();

    public PvPClassManager() {
        ArcherClass archerClass = new ArcherClass();

        this.pvpClasses.add(archerClass);

        BardClass bardClass = new BardClass();

        this.pvpClasses.add(bardClass);

        TankClass tankClass = new TankClass();

        this.pvpClasses.add(tankClass);

        for (PvPClass pvpClass : this.pvpClasses) {
            this.main.getServer().getPluginManager().registerEvents(pvpClass, this.main);
        }

        this.main.getServer().getScheduler().runTaskTimer(this.main, () -> {
            for (Player player : this.main.getServer().getOnlinePlayers()) {
                if (this.equippedPvPClasses.containsKey(player.getUniqueId())) {
                    PvPClass pvpClass = this.equippedPvPClasses.get(player.getUniqueId());

                    if (!pvpClass.qualifies(player)) {
                        this.equippedPvPClasses.remove(player.getUniqueId());

                        pvpClass.remove(player);
                        pvpClass.removeInfiniteEffects(player);

                        player.sendMessage(Utils.translate(Constants.PREFIX + "You have disabled the " + pvpClass.getName() + " class"));
                    } else if (!player.hasMetadata("Player frozen")) {
                        pvpClass.tick(player);
                    }
                } else {
                    for (PvPClass pvpClass : this.pvpClasses) {
                        if (pvpClass.qualifies(player) && pvpClass.canApply(player) && !player.hasMetadata("Player frozen")) {
                            this.equippedPvPClasses.put(player.getUniqueId(), pvpClass);

                            pvpClass.apply(player);

                            player.sendMessage(Utils.translate(Constants.PREFIX + "You have enabled the " + pvpClass.getName() + " class"));

                            break;
                        }
                    }
                }
            }

            this.lookAtSavedPotionEffects();
        }, 2L, 2L);
    }

    public PvPClass getPvPClass(Player player) {
        return this.equippedPvPClasses.getOrDefault(player.getUniqueId(), null);
    }

    public boolean hasPvPClassOn(Player player, PvPClass pvpClass) {
        if (!this.equippedPvPClasses.containsKey(player.getUniqueId())) {
            return false;
        }

        return this.equippedPvPClasses.get(player.getUniqueId()) == pvpClass;
    }

    public void lookAtSavedPotionEffects() {
        Iterator<Map.Entry<UUID, SavedPotionEffect>> iteratorOne = this.savedPotionEffects.entrySet().iterator();

        while (iteratorOne.hasNext()) {
            Map.Entry<UUID, SavedPotionEffect> iteratorTwo = iteratorOne.next();

            Player player = this.main.getServer().getPlayer(iteratorTwo.getKey());

            if (player != null) {
                if (System.currentTimeMillis() > iteratorTwo.getValue().getTime() && !iteratorTwo.getValue().isPermanent()) {
                    if (player.hasPotionEffect(iteratorTwo.getValue().getPotionEffect().getType())) {
                        player.getActivePotionEffects().forEach(activePotionEffect -> {
                            PotionEffect potionEffect = iteratorTwo.getValue().getPotionEffect();

                            if (potionEffect.getType() == activePotionEffect.getType() && potionEffect.getDuration() > activePotionEffect.getDuration() && potionEffect.getAmplifier() >= activePotionEffect.getAmplifier()) {
                                player.removePotionEffect(potionEffect.getType());
                            }
                        });
                    }

                    if (player.addPotionEffect(iteratorTwo.getValue().getPotionEffect(), true)) {
                        iteratorOne.remove();
                    }
                }
            } else {
                iteratorOne.remove();
            }
        }
    }

    @EventHandler
    public void onPlayerJoin(PlayerJoinEvent event) {
        Player player = event.getPlayer();

        if (this.equippedPvPClasses.containsKey(player.getUniqueId())) {
            this.equippedPvPClasses.get(player.getUniqueId()).remove(player);
            this.equippedPvPClasses.remove(player.getUniqueId());
        }

        for (PotionEffect potionEffect : player.getActivePotionEffects()) {
            if (potionEffect.getDuration() > 1_000_000) {
                player.removePotionEffect(potionEffect.getType());
            }
        }
    }

    @EventHandler
    public void onPlayerQuit(PlayerQuitEvent event) {
        Player player = event.getPlayer();

        if (this.equippedPvPClasses.containsKey(player.getUniqueId())) {
            this.equippedPvPClasses.get(player.getUniqueId()).remove(player);
            this.equippedPvPClasses.remove(player.getUniqueId());
        }
    }

    @EventHandler
    public void onPlayerKick(PlayerKickEvent event) {
        Player player = event.getPlayer();

        if (this.equippedPvPClasses.containsKey(player.getUniqueId())) {
            this.equippedPvPClasses.get(player.getUniqueId()).remove(player);
            this.equippedPvPClasses.remove(player.getUniqueId());
        }
    }

    @EventHandler
    public void onPlayerInteract(PlayerInteractEvent event) {
        Player player = event.getPlayer();

        if ((event.getAction() != Action.RIGHT_CLICK_AIR && event.getAction() != Action.RIGHT_CLICK_BLOCK) || player.getItemInHand() == null) {
            return;
        }

        for (PvPClass pvpClass : this.pvpClasses) {
            if (pvpClass.getConsumables() == null) {
                continue;
            }

            if (this.hasPvPClassOn(player, pvpClass) && pvpClass.getConsumables().contains(player.getItemInHand().getType())) {
                if (pvpClass.isItemConsumed(player, event.getItem().getType())) {
                    if (player.getItemInHand().getAmount() > 1) {
                        player.getItemInHand().setAmount(player.getItemInHand().getAmount() - 1);
                    } else {
                        player.getInventory().remove(player.getItemInHand());
                    }
                }
            }
        }
    }
}
