package me.trixkz.lifestealcore.managers.pvpclass;

import me.trixkz.lifestealcore.LifeStealCore;
import me.trixkz.lifestealcore.playerdata.PlayerData;
import me.trixkz.lifestealcore.playerdata.PlayerState;
import me.trixkz.lifestealcore.utils.Constants;
import me.trixkz.lifestealcore.utils.TimeUtils;
import me.trixkz.lifestealcore.utils.Utils;
import org.bukkit.Effect;
import org.bukkit.Material;
import org.bukkit.entity.Player;
import org.bukkit.event.EventHandler;
import org.bukkit.event.entity.EntityDamageByEntityEvent;
import org.bukkit.potion.PotionEffect;
import org.bukkit.potion.PotionEffectType;
import org.bukkit.scheduler.BukkitRunnable;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;
import java.util.UUID;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicInteger;

public class TankClass extends PvPClass {

    private LifeStealCore main = LifeStealCore.getInstance();

    private Map<UUID, AtomicInteger> playerHits = new HashMap<UUID, AtomicInteger>();
    private Map<UUID, Long> bleedingPlayers = new HashMap<UUID, Long>();
    private Map<UUID, Long> lastResistanceUsage = new HashMap<UUID, Long>();

    private int bleedingSeconds = 5;

    public TankClass() {
        super("Tank", 5, Arrays.asList(Material.IRON_INGOT));
    }

    @Override
    public void apply(Player player) {
        player.addPotionEffect(new PotionEffect(PotionEffectType.SPEED, Integer.MAX_VALUE, 1));
        player.addPotionEffect(new PotionEffect(PotionEffectType.DAMAGE_RESISTANCE, Integer.MAX_VALUE, 1));
    }

    @Override
    public void tick(Player player) {
        if (!player.hasPotionEffect(PotionEffectType.SPEED)) {
            player.addPotionEffect(new PotionEffect(PotionEffectType.SPEED, Integer.MAX_VALUE, 1));
        }

        if (!player.hasPotionEffect(PotionEffectType.DAMAGE_RESISTANCE)) {
            player.addPotionEffect(new PotionEffect(PotionEffectType.DAMAGE_RESISTANCE, Integer.MAX_VALUE, 1));
        }
    }

    @Override
    public void remove(Player player) {

    }

    @Override
    public boolean isItemConsumed(Player player, Material material) {
        PlayerData playerData = this.main.getPlayerDataManager().getPlayerData(player.getUniqueId());

        if (playerData.getPlayerState() != PlayerState.WARZONE) {
            player.sendMessage(Utils.translate(Constants.PREFIX + "You can only use this in the warzone"));

            return false;
        }

        if (material == Material.IRON_INGOT) {
            if (this.lastResistanceUsage.containsKey(player.getUniqueId())) {
                if (this.lastResistanceUsage.get(player.getUniqueId()) > System.currentTimeMillis()) {
                    long amountOfTimeLeft = this.lastResistanceUsage.get(player.getUniqueId()) - System.currentTimeMillis();

                    player.sendMessage(Utils.translate(Constants.PREFIX + "You can not use this for another " + TimeUtils.formatIntoDetailedString((int) amountOfTimeLeft / 1000)));

                    return false;
                }
            }

            this.lastResistanceUsage.put(player.getUniqueId(), System.currentTimeMillis() + TimeUnit.SECONDS.toMillis(30));

            player.addPotionEffect(new PotionEffect(PotionEffectType.DAMAGE_RESISTANCE, 100, 2));

            return true;
        }

        return false;
    }

    @Override
    public boolean qualifies(Player player) {
        return this.hasAllArmor(player) && player.getInventory().getHelmet().getType() == Material.DIAMOND_HELMET && player.getInventory().getChestplate().getType() == Material.IRON_CHESTPLATE && player.getInventory().getLeggings().getType() == Material.IRON_LEGGINGS && player.getInventory().getBoots().getType() == Material.DIAMOND_BOOTS;
    }

    @EventHandler
    public void onEntityDamageByEntity(EntityDamageByEntityEvent event) {
        Player killerPlayer = (Player) event.getDamager();

        PlayerData playerData = this.main.getPlayerDataManager().getPlayerData(killerPlayer.getUniqueId());

        if (playerData.getPlayerState() != PlayerState.WARZONE) {
            return;
        }

        Player victimPlayer = (Player) event.getEntity();

        if (this.main.getPvpClassManager().hasPvPClassOn(killerPlayer, this)) {
            if (this.playerHits.containsKey(killerPlayer.getUniqueId())) {
                this.playerHits.get(killerPlayer.getUniqueId()).getAndAdd(1);
            } else {
                AtomicInteger atomicInteger = new AtomicInteger();

                this.playerHits.put(killerPlayer.getUniqueId(), atomicInteger);
            }
        }

        if (this.playerHits.containsKey(killerPlayer.getUniqueId())) {
            if (this.playerHits.get(killerPlayer.getUniqueId()).get() >= 10) {
                this.playerHits.remove(killerPlayer.getUniqueId());
                this.bleedingPlayers.put(victimPlayer.getUniqueId(), System.currentTimeMillis() + (this.bleedingSeconds * 1000));

                new BukkitRunnable() {
                    int seconds = 0;

                    @Override
                    public void run() {
                        if (this.seconds == 100) {
                            this.cancel();

                            return;
                        }

                        victimPlayer.playEffect(victimPlayer.getLocation(), Effect.MOBSPAWNER_FLAMES, 50);
                        victimPlayer.damage(1);

                        this.seconds++;
                    }
                }.runTaskTimer(this.main, 20L, 100L);

                killerPlayer.sendMessage(Utils.translate("&8[&9Bleeding&8] &eYou have given " + victimPlayer.getName() + " the bleeding effect for 5 seconds"));
                victimPlayer.sendMessage(Utils.translate("&8[&9Bleeding&8] &eA tank has given you the bleeding effect for 5 seconds"));
            }
        }
    }
}
