package me.trixkz.lifestealcore.runnables;

import eu.decentsoftware.holograms.api.DHAPI;
import eu.decentsoftware.holograms.api.DecentHologramsAPI;
import eu.decentsoftware.holograms.api.holograms.Hologram;
import lombok.Getter;
import lombok.Setter;
import me.trixkz.lifestealcore.LifeStealCore;
import me.trixkz.lifestealcore.utils.Constants;
import me.trixkz.lifestealcore.utils.Utils;
import org.bukkit.Effect;
import org.bukkit.Location;
import org.bukkit.Material;
import org.bukkit.block.Block;
import org.bukkit.entity.Player;
import org.bukkit.inventory.ItemStack;
import org.bukkit.scheduler.BukkitRunnable;
import org.bukkit.util.Vector;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ThreadLocalRandom;

@Getter
@Setter
public class LootboxRunnable extends BukkitRunnable {

    private LifeStealCore main = LifeStealCore.getInstance();

    private Player player;

    private Block block;

    private List<ItemStack> itemStacks = new ArrayList<ItemStack>();
    private List<Location> locations = new ArrayList<Location>();

    private Map<Location, Integer> hologramTaskIds = new HashMap<Location, Integer>();

    private int itemStackIndex = 0;
    private int locationIndex = 0;
    private int seconds = 0;

    private boolean done = false;
    private boolean doneAlmost = false;

    public LootboxRunnable(Player player, Block block) {
        this.player = player;
        this.block = block;
        this.itemStacks = this.main.getLootboxManager().getItemStacks();

        Location location = this.block.getLocation().clone();
        Location locationCopy = location.clone();
        locationCopy.add(0.5, 0.5, 0.5);
        locationCopy.setY(locationCopy.getY() + 5);

        Hologram hologram = new Hologram("Hologram", locationCopy);

        DecentHologramsAPI.get().getHologramManager().registerHologram(hologram);

        this.hologramTaskIds.put(locationCopy, this.main.getServer().getScheduler().scheduleSyncRepeatingTask(this.main, new Runnable() {
            @Override
            public void run() {
                DHAPI.removeHologramLine(hologram, 0);
                DHAPI.removeHologramLine(hologram, 1);
                DHAPI.addHologramLine(hologram, Utils.translate("&d&l" + player.getName() + "&e's Lootbox"));
                DHAPI.addHologramLine(hologram, Utils.translate("&9Obtained in crates or at &d" + Constants.WEBSITE));
            }
        }, 0, 100));

//        World world = this.block.getLocation().getWorld();
//
//        BlockPosition position = new BlockPosition(this.block.getLocation().getX(), this.block.getLocation().getY(), this.block.getLocation().getZ());
//
//        TileEntityEnderChest tileEntityEnderChest = (TileEntityEnderChest) world.getTileEntity(position);
//
//        world.playBlockAction(position, tileEntityEnderChest.w(), 1, 1);

        for (int i = 0; i < 270; i += 45) {
            Location locationCopyNew = this.getHorizontalLocationAroundCenterLocation(this.block.getLocation(), 2, i).toLocation(this.player.getWorld());

            this.locations.add(locationCopyNew);
        }
    }

    @Override
    public void run() {
        if (this.doneAlmost) {
            if (this.locations.size() == this.locationIndex) {
                this.done = true;
                this.doneAlmost = false;

                return;
            }

            Location location = this.locations.get(this.locationIndex).clone();
            location.add(0.5, 0.5, 0.5);
            location.setY(location.getY() + 2);

            this.main.getServer().getScheduler().cancelTask(this.hologramTaskIds.get(location));
            this.locations.get(this.locationIndex).getWorld().strikeLightning(location);
            this.hologramTaskIds.remove(location);

            for (Hologram hologram : DecentHologramsAPI.get().getHologramManager().getHolograms()) {
                if (location.getWorld().getName().equalsIgnoreCase(hologram.getLocation().getWorld().getName()) && location.getX() == hologram.getLocation().getX() && location.getY() == hologram.getLocation().getY() && location.getZ() == hologram.getLocation().getZ()) {
                    hologram.delete();

                    break;
                }
            }

            this.locationIndex++;

            return;
        }

        if (this.done) {
            if (this.seconds == 1) {
                this.main.getServer().getScheduler().runTaskLater(this.main, () -> {
                    this.block.setType(Material.AIR);
                    this.block.getLocation().getWorld().strikeLightning(this.block.getLocation());
                    this.makeHelixParticleEffect(this.block.getLocation().clone(), Effect.SMOKE);
                }, 50L);

                this.main.getServer().getScheduler().runTaskLaterAsynchronously(this.main, () -> {
                    for (Hologram hologramToRemove : DecentHologramsAPI.get().getHologramManager().getHolograms()) {
                        if (this.hologramTaskIds.containsKey(hologramToRemove.getLocation())) {
                            hologramToRemove.delete();
                        }
                    }

                    this.hologramTaskIds.forEach((location, taskId) -> {
                        this.main.getServer().getScheduler().cancelTask(taskId);
                    });

                    this.cancel();
                }, 50L);
            }

            this.seconds++;

            return;
        }

        if (this.locations.size() == this.locationIndex) {
            this.locationIndex = 0;
        }

        Location location = this.locations.get(this.locationIndex).clone();
        location.add(0.5, 0.5, 0.5);
        location.setY(location.getY() + 2);

        for (Hologram hologram : DecentHologramsAPI.get().getHologramManager().getHolograms()) {
            if (location.getWorld().getName().equalsIgnoreCase(hologram.getLocation().getWorld().getName()) && location.getX() == hologram.getLocation().getX() && location.getY() == hologram.getLocation().getY() && location.getZ() == hologram.getLocation().getZ()) {
                this.main.getServer().getScheduler().cancelTask(this.hologramTaskIds.get(location));
                this.hologramTaskIds.remove(location);

                hologram.delete();

                break;
            }
        }

        if ((this.itemStacks.size() - 1) >= this.itemStackIndex) {
            this.spawnHologram(this.locations.get(this.locationIndex).clone(), this.itemStacks.get(this.itemStackIndex));
            this.itemStackIndex++;
        } else {
            this.itemStackIndex = 0;
            this.spawnHologram(this.locations.get(this.locationIndex).clone(), this.itemStacks.get(this.itemStackIndex));
            this.itemStackIndex++;
        }

        this.locationIndex++;
        this.seconds++;

        if (this.seconds >= 125) {
            int random = ThreadLocalRandom.current().nextInt(0, this.itemStacks.size());

            this.player.sendMessage(Utils.translate("&eYou have won &d" + this.itemStacks.get(random).getItemMeta().getDisplayName()));
            this.player.getInventory().addItem(this.itemStacks.get(random));

            Location locationCopy = this.block.getLocation().clone();
            Location locationCopyNew = locationCopy.clone();
            locationCopyNew.add(0.5, 0.5, 0.5);
            locationCopyNew.setY(locationCopyNew.getY() + 3);

            Hologram hologram = new Hologram("Hologram", locationCopyNew);

            DecentHologramsAPI.get().getHologramManager().registerHologram(hologram);

            this.hologramTaskIds.put(locationCopyNew, this.main.getServer().getScheduler().scheduleSyncRepeatingTask(this.main, new Runnable() {
                @Override
                public void run() {
                    DHAPI.removeHologramLine(hologram, 0);
                    DHAPI.removeHologramLine(hologram, 1);
                    DHAPI.removeHologramLine(hologram, 2);
                    DHAPI.addHologramLine(hologram, itemStacks.get(random));
                    DHAPI.addHologramLine(hologram, Utils.translate(""));
                    DHAPI.addHologramLine(hologram, Utils.translate(itemStacks.get(random).getItemMeta().getDisplayName()));
                }
            }, 0, 100));

            this.locationIndex = 0;
            this.seconds = 0;
            this.doneAlmost = true;
        }
    }

    public Vector getHorizontalLocationAroundCenterLocation(Location center, double radius, double angleInRadian) {
        double x = center.getX() + radius * Math.cos(angleInRadian);
        double y = center.getY();
        double z = center.getZ() + radius * Math.sin(angleInRadian);

        Vector vec = new Vector(x, y, z);

        return vec;
    }

    public Vector getVerticalLocationAroundCenterLocation(Location center, double radius, double angleInRadian) {
        double x = center.getX() + radius * Math.cos(angleInRadian);
        double y = center.getY() + radius * Math.sin(angleInRadian);
        double z = center.getZ();

        Vector vec = new Vector(x, y, z);

        return vec;
    }

    public void spawnHologram(Location location, ItemStack itemStack) {
        location.add(0.5, 0.5, 0.5);
        location.setY(location.getY() + 2);

        Hologram hologram = new Hologram("Hologram", location);

        DecentHologramsAPI.get().getHologramManager().registerHologram(hologram);

        this.hologramTaskIds.put(location, this.main.getServer().getScheduler().scheduleSyncRepeatingTask(this.main, new Runnable() {
            @Override
            public void run() {
                DHAPI.removeHologramLine(hologram, 0);
                DHAPI.removeHologramLine(hologram, 1);
                DHAPI.removeHologramLine(hologram, 2);
                DHAPI.addHologramLine(hologram, itemStack);
                DHAPI.addHologramLine(hologram, Utils.translate(""));
                DHAPI.addHologramLine(hologram, Utils.translate(itemStack.getItemMeta().getDisplayName()));
            }
        }, 0, 100));
    }

    public void makeHelixParticleEffect(Location location, Effect e) {
        location.add(0.5, 0.5, 0.5);

        new BukkitRunnable() {
            double t = 0;
            double r = 1;
            double up = 0.2;

            @Override
            public void run() {
                t = t + Math.PI/8;
                double x = r*Math.cos(t);
                double y = up*t;
                double z = r*Math.sin(t);
                location.add(x, y, z);
                location.getWorld().playEffect(location, e, 1);
                location.subtract(x, y, z);

                if (t > Math.PI*8) {
                    this.cancel();
                }
            }
        }.runTaskTimer(this.main, 0, 1);
    }
}
