package me.trixkz.lifestealcore.runnables;

import me.trixkz.lifestealcore.LifeStealCore;
import me.trixkz.lifestealcore.utils.Cuboid;
import me.trixkz.lifestealcore.utils.Utils;
import org.bukkit.Location;
import org.bukkit.Particle;
import org.bukkit.World;
import org.bukkit.block.Block;
import org.bukkit.entity.Player;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.UUID;

public class SpawnRunnable {

    private LifeStealCore main = LifeStealCore.getInstance();

    public SpawnRunnable() {
        this.main.getServer().getScheduler().runTaskTimerAsynchronously(this.main, () -> {
            for (Map.Entry<UUID, Integer> mapInformation : this.main.getSpawnManager().getPlayers().entrySet()) {
                Player player = this.main.getServer().getPlayer(mapInformation.getKey());

                if (player != null) {
                    if (this.main.getCombatManager().isInCombat(player.getUniqueId())) {
                        this.main.getSpawnManager().getPlayers().remove(mapInformation.getKey());

                        player.sendMessage(Utils.translate("&cYou can not go to spawn while you are in combat"));
                    } else {
                        if (mapInformation.getValue() <= 0) {
                            this.main.getSpawnManager().getPlayers().remove(player.getUniqueId());
                            this.main.getPlayerDataManager().sendToSpawnAndResetPlayer(player);

                            player.sendMessage(Utils.translate("&aTeleporting to spawn..."));
                        } else {
                            this.main.getSpawnManager().getPlayers().put(mapInformation.getKey(), mapInformation.getValue() - 1);

                            player.sendMessage(Utils.translate("&aTeleporting to spawn in " + mapInformation.getValue() + "..."));
                        }
                    }
                } else {
                    this.main.getSpawnManager().getPlayers().remove(mapInformation.getKey());
                }
            }

            if (this.main.getSpawnManager().getSpawnMin() != null) {
                Cuboid cuboid = new Cuboid(this.main.getSpawnManager().getCornerMin().toBukkitLocation(), this.main.getSpawnManager().getCornerMax().toBukkitLocation());

                for (Player player : this.main.getServer().getOnlinePlayers()) {
                    for (Block block : cuboid.getWalls()) {
                        Location location = block.getLocation();

                        player.spawnParticle(Particle.GLOW, location, 1);
                    }
                }
            }
        }, 0L, 20L);
    }

    public List<Location> getAreaOutline(Location corner1, Location corner2, double particleDistance) {
        List<Location> result = new ArrayList<Location>();
        World world = corner1.getWorld();
        double minX = Math.min(corner1.getX(), corner2.getX());
        double minY = Math.min(corner1.getY(), corner2.getY());
        double minZ = Math.min(corner1.getZ(), corner2.getZ());
        double maxX = Math.max(corner1.getX(), corner2.getX());
        double maxY = Math.max(corner1.getY(), corner2.getY());
        double maxZ = Math.max(corner1.getZ(), corner2.getZ());

        for (double x = minX; x <= maxX; x+=particleDistance) {
            for (double y = minY; y <= maxY; y+=particleDistance) {
                for (double z = minZ; z <= maxZ; z+=particleDistance) {
                    int components = 0;
                    if (x == minX || x == maxX) components++;
                    if (y == minY || y == maxY) components++;
                    if (z == minZ || z == maxZ) components++;
                    if (components >= 2) {
                        result.add(new Location(world, x, y, z));
                    }
                }
            }
        }

        return result;
    }
}
