package me.trixkz.lifestealcore.scoreboard;

import aether.scoreboard.Board;
import aether.scoreboard.BoardAdapter;
import aether.scoreboard.cooldown.BoardCooldown;
import me.trixkz.lifestealcore.LifeStealCore;
import me.trixkz.lifestealcore.playerdata.PlayerData;
import me.trixkz.lifestealcore.utils.Constants;
import me.trixkz.lifestealcore.utils.Utils;
import org.bukkit.entity.Player;
import org.bukkit.scoreboard.Scoreboard;

import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;

public class ScoreboardProvider implements BoardAdapter {

    private LifeStealCore main = LifeStealCore.getInstance();

    @Override
    public String getTitle(Player player) {
        // ColorSet<Integer, Integer, Integer> colorSetOne = new ColorSet<>(ColorUtils.getColorMap().get(ChatColor.valueOf(this.main.getMessagesConfig().getConfig().getString("SCOREBOARD.NAME-COLOR-CODE-ONE"))).getRed(), ColorUtils.getColorMap().get(ChatColor.valueOf(this.main.getMessagesConfig().getConfig().getString("SCOREBOARD.NAME-COLOR-CODE-ONE"))).getGreen(), ColorUtils.getColorMap().get(ChatColor.valueOf(this.main.getMessagesConfig().getConfig().getString("SCOREBOARD.NAME-COLOR-CODE-ONE"))).getBlue());
        // ColorSet<Integer, Integer, Integer> colorSetTwo = new ColorSet<>(ColorUtils.getColorMap().get(ChatColor.valueOf(this.main.getMessagesConfig().getConfig().getString("SCOREBOARD.NAME-COLOR-CODE-TWO"))).getRed(), ColorUtils.getColorMap().get(ChatColor.valueOf(this.main.getMessagesConfig().getConfig().getString("SCOREBOARD.NAME-COLOR-CODE-TWO"))).getGreen(), ColorUtils.getColorMap().get(ChatColor.valueOf(this.main.getMessagesConfig().getConfig().getString("SCOREBOARD.NAME-COLOR-CODE-TWO"))).getBlue());

        // return ColorUtils.getGradientString(this.main.getMessagesConfig().getConfig().getString("SCOREBOARD.NAME"), ColorUtils.getColorCodes(this.main.getMessagesConfig().getConfig().getString("SCOREBOARD.NAME"), colorSetOne, colorSetTwo));

        return Utils.translate(this.main.getMessagesConfig().getConfig().getString("SCOREBOARD.NAME"));
    }

    @Override
    public List<String> getScoreboard(Player player, Board board, Set<BoardCooldown> cooldowns) {
        PlayerData playerData = this.main.getPlayerDataManager().getPlayerData(player.getUniqueId());

        if (playerData == null) {
            return null;
        }

        if (!playerData.getPlayerSettings().isScoreboardEnabled()) {
            return null;
        }

        switch (playerData.getPlayerState()) {
            case SPAWN:
                return this.getSpawnScoreboard(player, playerData);
            case WARZONE:
                return this.getWarzoneScoreboard(player, playerData);
        }

        return null;
    }

    @Override
    public void onScoreboardCreate(Player player, Scoreboard scoreboard) {

    }

    private List<String> getSpawnScoreboard(Player player, PlayerData playerData) {
        List<String> lines = new ArrayList<>();

        DecimalFormat decimalFormat = new DecimalFormat("##.##");

        for (String string : Constants.SPAWN_SCOREBOARD) {
            lines.add(
                    string
                            .replaceAll("%scoreboard_bar%", String.valueOf(Utils.scoreboardBar))
                            .replaceAll("%players_online%", String.valueOf(this.main.getServer().getOnlinePlayers().size()))
                            .replaceAll("%player%", player.getName())
                            .replaceAll("%coins%", String.valueOf(decimalFormat.format(playerData.getCoins())))
                            .replaceAll("%tokens%", String.valueOf(playerData.getTokens()))
                            .replaceAll("%website%", Constants.WEBSITE)
            );
        }

        return Utils.translate(lines);
    }

    private List<String> getWarzoneScoreboard(Player player, PlayerData playerData) {
        List<String> lines = new ArrayList<>();

        DecimalFormat decimalFormat = new DecimalFormat("##.##");

        for (String string : Constants.WARZONE_SCOREBOARD) {
            lines.add(
                    string
                            .replaceAll("%scoreboard_bar%", String.valueOf(Utils.scoreboardBar))
                            .replaceAll("%players_online%", String.valueOf(this.main.getServer().getOnlinePlayers().size()))
                            .replaceAll("%player%", player.getName())
                            .replaceAll("%kills%", String.valueOf(playerData.getKills()))
                            .replaceAll("%killstreak%", String.valueOf(playerData.getCurrentKillstreak()))
                            .replaceAll("%deaths%", String.valueOf(playerData.getDeaths()))
                            .replaceAll("%combat_tag%", this.main.getCombatManager().getCombatTime(player.getUniqueId()) + "&ds")
                            .replaceAll("%pvp_class%", String.valueOf(this.main.getPvpClassManager().getPvPClass(player).getName()))
                            .replaceAll("%website%", Constants.WEBSITE)
            );
        }

        return Utils.translate(lines);
    }
}
