package me.trixkz.lifestealcore.utils;

import me.trixkz.lifestealcore.LifeStealCore;
import org.bukkit.*;
import org.bukkit.block.Block;
import org.bukkit.entity.Player;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.TimeZone;
import java.util.concurrent.TimeUnit;
import java.util.function.Consumer;

public class Utils {

    private static LifeStealCore main = LifeStealCore.getInstance();

    public static String scoreboardBar = ChatColor.GRAY.toString() + ChatColor.STRIKETHROUGH + "----------------------";
    public static String chatBar = ChatColor.GRAY.toString() + ChatColor.STRIKETHROUGH + "--------------------------------------------";

    /**
     * Translate the message as a string
     *
     * @param message - The message
     * @return The translated message as a string
     */
    public static String translate(String message) {
        return ChatColor.translateAlternateColorCodes('&', message);
    }

    /**
     * Translate the message as an array list
     *
     * @param message - The message
     * @return The translated message as an array list
     */
    public static List<String> translate(List<String> lines) {
        List<String> strings = new ArrayList<>();
        for (String line : lines) {
            strings.add(ChatColor.translateAlternateColorCodes('&', line));
        }
        return strings;
    }

    /**
     * Translate the message as an array
     *
     * @param message - The message
     * @return The translated message as an array list
     */
    public static List<String> translate(String[] lines) {
        List<String> strings = new ArrayList<>();
        for (String line : lines) {
            if (line != null) {
                strings.add(ChatColor.translateAlternateColorCodes('&', line));
            }
        }
        return strings;
    }

    public static List<Block> getBlocks(CustomLocation loc1, CustomLocation loc2, World w) {
        //First of all, we create the list:
        List<Block> blocks = new ArrayList<Block>();

        //Next we will name each coordinate
        int x1 = loc1.toBukkitLocation().getBlockX();
        int y1 = loc1.toBukkitLocation().getBlockY();
        int z1 = loc1.toBukkitLocation().getBlockZ();

        int x2 = loc2.toBukkitLocation().getBlockX();
        int y2 = loc2.toBukkitLocation().getBlockY();
        int z2 = loc2.toBukkitLocation().getBlockZ();

        //Then we create the following integers
        int xMin, yMin, zMin;
        int xMax, yMax, zMax;
        int x, y, z;

        //Now we need to make sure xMin is always lower then xMax
        if(x1 > x2){ //If x1 is a higher number then x2
            xMin = x2;
            xMax = x1;
        }else{
            xMin = x1;
            xMax = x2;
        }

        //Same with Y
        if(y1 > y2){
            yMin = y2;
            yMax = y1;
        }else{
            yMin = y1;
            yMax = y2;
        }

        //And Z
        if(z1 > z2){
            zMin = z2;
            zMax = z1;
        }else{
            zMin = z1;
            zMax = z2;
        }

        //Now it's time for the loop
        for(x = xMin; x <= xMax; x ++){
            for(y = yMin; y <= yMax; y ++){
                for(z = zMin; z <= zMax; z ++){
                    Block b = new Location(w, x, y, z).getBlock();
                    blocks.add(b);
                }
            }
        }

        //And last but not least, we return with the list
        return blocks;
    }

    public static boolean getPlayer(Player p, CustomLocation loc1, CustomLocation loc2) {
        double x1 = loc1.toBukkitLocation().getX();
        double y1 = loc1.toBukkitLocation().getY();
        double z1 = loc1.toBukkitLocation().getZ();
        double x2 = loc2.toBukkitLocation().getX();
        double y2 = loc2.toBukkitLocation().getY();
        double z2 = loc2.toBukkitLocation().getZ();

        return (p.getLocation().getX() > x1) && (p.getLocation().getY() > y1) && (p.getLocation().getZ() > z1) && (p.getLocation().getX() < x2) && (p.getLocation().getY() < y2) && (p.getLocation().getZ() < z2);
    }

    public static void getOfflinePlayer(String player, Consumer<OfflinePlayer> consumer) {
        //Do async as this can block the main thread
        AsyncScheduler.run(() -> {
            OfflinePlayer offlinePlayer = Bukkit.getOfflinePlayer(player);

            consumer.accept(offlinePlayer);
        });
    }

    public static List<Block> getBlocks(Location location, int radius) {
        List<Block> blocks = new ArrayList<Block>();
        for(int x = location.getBlockX() - radius; x <= location.getBlockX() + radius; x++) {
            for(int y = location.getBlockY() - radius; y <= location.getBlockY() + radius; y++) {
                for(int z = location.getBlockZ() - radius; z <= location.getBlockZ() + radius; z++) {
                    blocks.add(location.getWorld().getBlockAt(x, y, z));
                }
            }
        }
        return blocks;
    }

    public static String makeTimeReadable(Long time) {
        if (time == null)
            return "";
        StringBuilder sb = new StringBuilder();
        long days = TimeUnit.MILLISECONDS.toDays(time);
        long hours = TimeUnit.MILLISECONDS.toHours(time) - TimeUnit.DAYS.toHours(TimeUnit.MILLISECONDS.toDays(time));
        long minutes = TimeUnit.MILLISECONDS.toMinutes(time) - TimeUnit.HOURS.toMinutes(TimeUnit.MILLISECONDS.toHours(time));
        long seconds = TimeUnit.MILLISECONDS.toSeconds(time) - TimeUnit.MINUTES.toSeconds(TimeUnit.MILLISECONDS.toMinutes(time));
        if (days != 0L)
            sb.append(" ").append(days).append("d");
        if (hours != 0L)
            sb.append(" ").append(hours).append("h");
        if (minutes != 0L)
            sb.append(" ").append(minutes).append("m");
        if (seconds != 0L)
            sb.append(" ").append(seconds).append("s");
        return sb.toString().trim();
    }

    public static long parseTime(String input) {
        long result = 0L;
        StringBuilder number = new StringBuilder();
        for (int i = 0; i < input.length(); i++) {
            char c = input.charAt(i);
            if (Character.isDigit(c)) {
                number.append(c);
            } else if (Character.isLetter(c) && number.length() > 0) {
                result += convert(Integer.parseInt(number.toString()), c);
                number = new StringBuilder();
            }
        }
        return result;
    }

    private static long convert(long value, char unit) {
        switch (unit) {
            case 'd':
                return value * 1000L * 60L * 60L * 24L;
            case 'h':
                return value * 1000L * 60L * 60L;
            case 'm':
                return value * 1000L * 60L;
            case 's':
                return value * 1000L;
        }
        return 0L;
    }

    public static String getAddedAtDate(long addedAt) {
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("dd/MM/yyyy hh:mm:ss");
        simpleDateFormat.setTimeZone(TimeZone.getTimeZone("EST"));

        return simpleDateFormat.format(new Date(addedAt));
    }

    public static boolean isNumeric(String string) {
        return regexNumeric(string).length() == 0;
    }

    public static String regexNumeric(String string) {
        return string.replaceAll("[0-9]", "").replaceFirst("\\.", "");
    }
}
